@@ -4,7 +4,7 @@
 
 static void _end_weapon_brand()
 {
-    you.duration[DUR_WEAPON_BRAND] = 1;
+    you.duration[DUR_EXCRUCIATING_WOUNDS] = 1;
     ASSERT(you.weapon());
     end_weapon_brand(*you.weapon(), true);
 }
@@ -207,15 +207,10 @@ static const duration_def duration_data[] =
       "liquid flames", "",
       "You are covered in liquid flames.", D_NO_FLAGS},
     { DUR_LOWERED_MR,
-      RED, "MR-",
+      RED, "MR/2",
       "vulnerable", "lowered mr",
-      "", D_NO_FLAGS,
-      {{ "You feel less vulnerable to hostile enchantments." }}},
-    { DUR_MAGIC_SHIELD,
-      0, "",
-      "shielded", "magic shield",
       "", D_DISPELLABLE,
-      {{ "Your magical shield disappears.", _redraw_armour }}},
+      {{ "You feel less vulnerable to hostile enchantments." }}},
     { DUR_MIGHT,
       LIGHTBLUE, "Might",
       "mighty", "might",
@@ -251,24 +246,11 @@ static const duration_def duration_data[] =
     { DUR_SLEEP,
       0, "",
       "sleeping", "sleep",
-      "You are sleeping.", D_NO_FLAGS},
-    { DUR_MAGIC_ARMOUR,
-      0, "",
-      "magic armour", "magic armour",
-      "You are magically armoured.", D_DISPELLABLE,
-      {{ "Your magical armour fades away.", [](){
-          you.props.erase(MAGIC_ARMOUR_KEY);
-          you.redraw_armour_class = true;
-      }}}},
+      "You are sleeping.", D_DISPELLABLE},
     { DUR_SWIFTNESS,
       BLUE, "Swift",
       "swift", "swiftness",
       "You can move swiftly.", D_DISPELLABLE | D_EXPIRES, {}, 6},
-    { DUR_TELEPATHY,
-      LIGHTBLUE, "Emp",
-      "empathic", "telepathy",
-      "", D_NO_FLAGS,
-      {{ "You feel less empathic." }}},
     { DUR_TELEPORT,
       LIGHTBLUE, "Tele",
       "about to teleport", "teleport",
@@ -298,7 +280,7 @@ static const duration_def duration_data[] =
     { DUR_STEALTH,
       BLUE, "Stealth",
       "especially stealthy", "stealth",
-      "", D_DISPELLABLE,
+      "", D_NO_FLAGS,
       {{ "You feel less stealthy." }}},
     { DUR_AFRAID,
       RED, "Fear",
@@ -395,7 +377,7 @@ static const duration_def duration_data[] =
     { DUR_FLAYED,
       RED, "Flay",
       "flayed", "",
-      "You are covered in terrible wounds.", D_EXPIRES},
+      "You are covered in terrible wounds.", D_DISPELLABLE /* but special-cased */ | D_EXPIRES},
     { DUR_WEAK,
       RED, "Weak",
       "weakened", "weak",
@@ -426,8 +408,8 @@ static const duration_def duration_data[] =
       {{ "You feel less vulnerable to fire." }}},
     { DUR_BARBS,
       RED, "Barbs",
-      "manticore barbs", "",
-      "Manticore spikes are embedded in your body.", D_NO_FLAGS},
+      "barbed spikes", "",
+      "Barbed spikes are embedded in your body.", D_NO_FLAGS},
     { DUR_POISON_VULN,
       RED, "rP-",
       "poison vulnerable", "poison vulnerability",
@@ -508,15 +490,8 @@ static const duration_def duration_data[] =
     { DUR_CORROSION,
       RED, "Corr",
       "corroded", "corrosion",
-      "You are corroded.", D_NO_FLAGS,
+      "You are corroded.", D_DISPELLABLE,
       {{ "You are no longer corroded.", _end_corrosion }}},
-    { DUR_FORTITUDE,
-      LIGHTBLUE, "Fort",
-      "immense fortitude", "",
-      "You have immense fortitude and shrug off injury.", D_DISPELLABLE,
-      {{ "Your fortitude fades away.", []() {
-          notify_stat_change(STAT_STR, -10, true);
-      }}}},
     { DUR_HORROR,
       RED, "Horr",
       "horrified", "",
@@ -533,10 +508,12 @@ static const duration_def duration_data[] =
     { DUR_CLEAVE,
       LIGHTBLUE, "Cleave",
       "cleaving", "cleave",
-      "You are cleaving through your foes.", D_DISPELLABLE,
+      "You are cleaving through your foes.", D_NO_FLAGS,
       {{ "Your cleaving frenzy subsides." }}},
     { DUR_AMBROSIA, GREEN, "Ambros", "", "ambrosia",
       "You are regenerating under the effects of ambrosia.", D_DISPELLABLE },
+    { DUR_CHANNEL_ENERGY, LIGHTBLUE, "Channel", "", "channel",
+      "You are rapidly regenerating magical energy.", D_DISPELLABLE },
     { DUR_DEVICE_SURGE, WHITE, "Surge", "device surging", "device surge",
       "You have readied a device surge.", D_EXPIRES,
       {{ "Your device surge dissipates." },
@@ -549,10 +526,17 @@ static const duration_def duration_data[] =
           you.props.erase(NEXT_DOOM_HOUND_KEY);
       }}}},
     { DUR_VERTIGO, YELLOW, "Vertigo", "", "vertigo",
-      "Vertigo is making it harder to attack, cast, and dodge.", D_NO_FLAGS,
+      "Vertigo is making it harder to attack, cast, and dodge.", D_DISPELLABLE,
       {{ "The world stops spinning.", []() {
           you.redraw_evasion = true;
       }}}},
+    { DUR_SANGUINE_ARMOUR, LIGHTBLUE, "Blood",
+      "sanguine armour", "",
+      "Your shed blood clings to and protects you.", D_NO_FLAGS,
+        {{ "Your blood armour dries and flakes away.", _redraw_armour }}},
+    { DUR_SPWPN_PROTECTION, 0, "", "protection aura", "",
+      "Your weapon is exuding a protective aura.", D_NO_FLAGS,
+      {{ "", _redraw_armour }}},
 
     // The following are visible in wizmode only, or are handled
     // specially in the status lights and/or the % or @ screens.
@@ -572,14 +556,14 @@ static const duration_def duration_data[] =
       {{ "Your skin stops crawling." },
           { "Your skin is crawling a little less now.", 1}}, 6},
     { DUR_TRANSFORMATION, 0, "", "", "transformation", "", D_DISPELLABLE /*but special-cased*/, {}, 10},
-    { DUR_WEAPON_BRAND, 0, "", "", "weapon brand", "", D_DISPELLABLE,
+    { DUR_EXCRUCIATING_WOUNDS, 0, "", "", "excruciating wounds", "", D_DISPELLABLE,
       {{ "", _end_weapon_brand }}},
     { DUR_DEMONIC_GUARDIAN, 0, "", "", "demonic guardian", "", D_NO_FLAGS, {{""}}},
     { DUR_POWERED_BY_DEATH, 0, "", "", "pbd", "", D_NO_FLAGS},
     { DUR_GOURMAND, 0, "", "", "gourmand", "", D_NO_FLAGS},
         { DUR_REPEL_STAIRS_MOVE, 0, "", "", "repel stairs move", "", D_NO_FLAGS, {{""}}},
     { DUR_REPEL_STAIRS_CLIMB, 0, "", "", "repel stairs climb", "", D_NO_FLAGS, {{""}}},
-    { DUR_COLOUR_SMOKE_TRAIL, 0, "", "", "coloured smoke trail", "", D_NO_FLAGS},
+    { DUR_CLOUD_TRAIL, 0, "", "", "cloud trail", "", D_NO_FLAGS},
     { DUR_TIME_STEP, 0, "", "", "time step", "", D_NO_FLAGS},
     { DUR_ICEMAIL_DEPLETED, 0, "", "", "icemail depleted", "", D_NO_FLAGS,
       {{ "Your icy envelope is restored.", _redraw_armour }}},
@@ -589,8 +573,8 @@ static const duration_def duration_data[] =
     { DUR_WATER_HOLD, 0, "", "", "drowning", "", D_NO_FLAGS},
     { DUR_WATER_HOLD_IMMUNITY, 0, "", "", "drowning immunity", "", D_NO_FLAGS, {{""}}},
     { DUR_SLEEP_IMMUNITY, 0, "", "", "sleep immunity", "", D_NO_FLAGS, {{""}}},
-    { DUR_ELIXIR_HEALTH, 0, "", "", "elixir health", "", D_DISPELLABLE},
-    { DUR_ELIXIR_MAGIC, 0, "", "", "elixir magic", "", D_DISPELLABLE},
+    { DUR_ELIXIR_HEALTH, 0, "", "", "elixir health", "", D_NO_FLAGS},
+    { DUR_ELIXIR_MAGIC, 0, "", "", "elixir magic", "", D_NO_FLAGS},
     { DUR_TROGS_HAND, 0, "", "", "trogs hand", "", D_NO_FLAGS,
         {{"", trog_remove_trogs_hand},
           {"You feel the effects of Trog's Hand fading.", 1}}, 6},
@@ -599,6 +583,9 @@ static const duration_def duration_data[] =
     { DUR_COLLAPSE, 0, "", "", "collapse", "", D_NO_FLAGS },
     { DUR_BRAINLESS, 0, "", "", "brainless", "", D_NO_FLAGS },
     { DUR_CLUMSY, 0, "", "", "clumsy", "", D_NO_FLAGS },
+    { DUR_ANCESTOR_DELAY, 0, "", "", "ancestor delay", "", D_NO_FLAGS, {{""}}},
+    { DUR_NO_CAST, 0, "", "", "no cast", "", D_NO_FLAGS,
+      {{ "You regain access to your magic." }, {}, true }},
 
 #if TAG_MAJOR_VERSION == 34
     // And removed ones
@@ -631,5 +618,9 @@ static const duration_def duration_data[] =
         "antimagic", "",
         "You have trouble accessing your magic.", D_DISPELLABLE | D_EXPIRES,
         {{ "You regain control over your magic." }}, 27},
+    { DUR_TELEPATHY, 0, "", "", "old telepathy", "", D_NO_FLAGS},
+    { DUR_MAGIC_ARMOUR, 0, "", "", "old magic armour", "", D_NO_FLAGS},
+    { DUR_MAGIC_SHIELD, 0, "", "", "old magic shield", "", D_NO_FLAGS},
+    { DUR_FORTITUDE, 0, "", "", "old fortitude", "", D_NO_FLAGS},
 #endif
 };
