@@ -184,7 +184,15 @@ void mirror_damage_fineff::fire()
 
     if (att == MID_PLAYER)
     {
-        mpr("Your damage is reflected back at you!");
+        const monster* reflector = defender() ?
+                                   defender()->as_monster() : nullptr;
+        if (reflector)
+        {
+            mprf("%s reflects your damage back at you!",
+                 reflector->name(DESC_THE).c_str());
+        }
+        else
+            mpr("Your damage is reflected back at you!");
         ouch(damage, KILLED_BY_MIRROR_DAMAGE);
     }
     else if (def == MID_PLAYER)
@@ -197,13 +205,13 @@ void mirror_damage_fineff::fire()
         attack->hurt(&you, damage);
 
         if (attack->alive())
-            print_wounds(monster_by_mid(att));
+            print_wounds(*monster_by_mid(att));
 
         lose_piety(isqrt_ceil(damage));
     }
     else
     {
-        simple_monster_message(monster_by_mid(att), " suffers a backlash!");
+        simple_monster_message(*monster_by_mid(att), " suffers a backlash!");
         attack->hurt(defender(), damage);
     }
 }
@@ -281,8 +289,9 @@ void trj_spawn_fineff::fire()
             continue;
 
         if (monster *mons = mons_place(
-                              mgen_data(jelly, spawn_beh, trj, 0, 0, jpos,
-                                        foe, MG_DONT_COME, GOD_JIYVA)))
+                              mgen_data(jelly, spawn_beh, jpos, foe,
+                                        MG_DONT_COME, GOD_JIYVA)
+                              .set_summoned(trj, 0, 0)))
         {
             // Don't allow milking the Royal Jelly.
             mons->flags |= MF_NO_REWARD;
@@ -383,7 +392,7 @@ void starcursed_merge_fineff::fire()
         monster* mergee = monster_at(*ai);
         if (mergee && mergee->alive() && mergee->type == MONS_STARCURSED_MASS)
         {
-            simple_monster_message(mon,
+            simple_monster_message(*mon,
                     " shudders and is absorbed by its neighbour.");
             _do_merge_masses(mon, mergee);
             return;
@@ -427,7 +436,7 @@ void starcursed_merge_fineff::fire()
 
             if (moved)
             {
-                simple_monster_message(mon, " shudders and withdraws towards its neighbour.");
+                simple_monster_message(*mon, " shudders and withdraws towards its neighbour.");
                 mon->speed_increment -= 10;
             }
         }
@@ -513,9 +522,9 @@ void bennu_revive_fineff::fire()
     bool res_visible = you.see_cell(posn);
 
 
-    monster *newmons = create_monster(mgen_data(MONS_BENNU,
-                                                attitude, 0, 0, 0, posn, foe,
-                                                res_visible ? MG_DONT_COME : MG_NONE));
+    monster *newmons = create_monster(mgen_data(MONS_BENNU, attitude, posn, foe,
+                                                res_visible ? MG_DONT_COME
+                                                            : MG_NONE));
     if (newmons)
         newmons->props["bennu_revives"].get_byte() = revives + 1;
 }
