@@ -24,11 +24,13 @@
 #include "invent.h"
 #include "itemprop.h"
 #include "items.h"
+#include "item_use.h"
 #include "libutil.h"
 #include "macro.h"
 #include "message.h"
 #include "misc.h"
 #include "mutation.h"
+#include "nearby-danger.h"
 #include "notes.h"
 #include "options.h"
 #include "output.h"
@@ -42,9 +44,8 @@
 #include "xom.h"
 
 static void _eat_chunk(item_def& food);
-static void _eating(item_def &food);
 static void _describe_food_change(int hunger_increment);
-static bool _vampire_consume_corpse(int slot, bool invent);
+static bool _vampire_consume_corpse(item_def& corpse);
 static void _heal_from_food(int hp_amt);
 
 void make_hungry(int hunger_amount, bool suppress_msg,
@@ -87,14 +88,6 @@ void make_hungry(int hunger_amount, bool suppress_msg,
         _describe_food_change(-hunger_amount);
 }
 
-// Must match the order of hunger_state_t enums
-static constexpr int hunger_threshold[HS_ENGORGED + 1] =
-{
-    HUNGER_FAINTING, HUNGER_STARVING, HUNGER_NEAR_STARVING, HUNGER_VERY_HUNGRY,
-    HUNGER_HUNGRY, HUNGER_SATIATED, HUNGER_FULL, HUNGER_VERY_FULL,
-    HUNGER_ENGORGED
-};
-
 /**
  * Attempt to reduce the player's hunger.
  *
@@ -157,43 +150,28 @@ bool you_foodless_normally()
         ;
 }
 
-bool prompt_eat_inventory_item(int slot)
+bool prompt_eat_item(int slot)
 {
-    // There's nothing in inventory that a vampire can 'e'.
+    // There's nothing in inventory that a vampire can 'e', and floor corpses
+    // are handled by prompt_eat_chunks.
     if (you.species == SP_VAMPIRE)
         return false;
 
-    if (inv_count() < 1)
-    {
-        canned_msg(MSG_NOTHING_CARRIED);
-        return false;
-    }
-
-    int which_inventory_slot = slot;
-
+    item_def* item = nullptr;
     if (slot == -1)
     {
-        which_inventory_slot = prompt_invent_item("Eat which item?",
-                                                  MT_INVLIST, OBJ_FOOD,
-                                                  true, true, true, 0, -1,
-                                                  nullptr, OPER_EAT);
-
-        if (prompt_failed(which_inventory_slot))
+        item = use_an_item(OBJ_FOOD, OPER_EAT, "Eat which item?");
+        if (!item)
             return false;
     }
+    else
+        item = &you.inv[slot];
 
-    item_def &item(you.inv[which_inventory_slot]);
-    if (item.base_type != OBJ_FOOD)
-    {
-        mpr("You can't eat that!");
-        return false;
-    }
-
-    if (!can_eat(item, false))
+    ASSERT(item);
+    if (!can_eat(*item, false))
         return false;
 
-    eat_item(item);
-    you.turn_is_over = true;
+    eat_item(*item);
 
     return true;
 }
@@ -236,26 +214,16 @@ bool eat_food(int slot)
     if (slot == -1)
     {
         int result = prompt_eat_chunks();
-        if (result == 1 || result == -1)
-            return result > 0;
-
-        if (result != -2) // else skip ahead to inventory
-        {
-            if (you.visible_igrd(you.pos()) != NON_ITEM)
-            {
-                result = eat_from_floor(true);
-                if (result == 1)
-                    return true;
-                if (result == -1)
-                    return false;
-            }
-        }
+        if (result == 1)
+            return true;
+        else if (result == -1)
+            return false;
     }
 
     if (you.species == SP_VAMPIRE)
         mpr("There's nothing here to drain!");
 
-    return prompt_eat_inventory_item(slot);
+    return prompt_eat_item(slot);
 }
 
 static string _how_hungry()
@@ -297,13 +265,6 @@ bool food_change(bool initial)
 
         if (you.species == SP_VAMPIRE)
         {
-            if (you.duration[DUR_BERSERK] > 1 && newstate <= HS_HUNGRY)
-            {
-                mprf(MSGCH_DURATION, "Your blood-deprived body can't sustain "
-                                     "your rage any longer.");
-                you.duration[DUR_BERSERK] = 1;
-            }
-
             switch (lifeless_prevents_form())
             {
             case UFR_TOO_DEAD:
@@ -421,23 +382,12 @@ static void _describe_food_change(int food_increment)
 
 bool eat_item(item_def &food)
 {
-    int link;
-
-    if (in_inventory(food))
-        link = food.link;
-    else
-    {
-        link = item_on_floor(food, you.pos());
-        if (link == NON_ITEM)
-            return false;
-    }
-
     if (food.is_type(OBJ_CORPSES, CORPSE_BODY))
     {
         if (you.species != SP_VAMPIRE)
             return false;
 
-        if (_vampire_consume_corpse(link, in_inventory(food)))
+        if (_vampire_consume_corpse(food))
         {
             count_action(CACT_EAT, -1); // subtype Corpse
             you.turn_is_over = true;
@@ -446,22 +396,125 @@ bool eat_item(item_def &food)
         else
             return false;
     }
-    else if (food.sub_type == FOOD_CHUNK)
+    else
+        start_delay<EatDelay>(food_turns(food) - 1, food);
+
+    mprf("You start eating %s%s.", food.quantity > 1 ? "one of " : "",
+                                   food.name(DESC_THE).c_str());
+    you.turn_is_over = true;
+    return true;
+}
+
+// Handle messaging at the end of eating.
+// Some food types may not get a message.
+static void _finished_eating_message(food_type type)
+{
+    bool herbivorous = player_mutation_level(MUT_HERBIVOROUS) > 0;
+    bool carnivorous = player_mutation_level(MUT_CARNIVOROUS) > 0;
+
+    if (herbivorous)
+    {
+        if (food_is_meaty(type))
+        {
+            mpr("Blech - you need greens!");
+            return;
+        }
+    }
+    else
+    {
+        switch (type)
+        {
+        case FOOD_MEAT_RATION:
+            mpr("That meat ration really hit the spot!");
+            return;
+        case FOOD_BEEF_JERKY:
+            mprf("That beef jerky was %s!",
+                 one_chance_in(4) ? "jerk-a-riffic"
+                                  : "delicious");
+            return;
+        default:
+            break;
+        }
+    }
+
+    if (carnivorous)
+    {
+        if (food_is_veggie(type))
+        {
+            mpr("Blech - you need meat!");
+            return;
+        }
+    }
+    else
+    {
+        switch (type)
+        {
+        case FOOD_BREAD_RATION:
+            mpr("That bread ration really hit the spot!");
+            return;
+        case FOOD_FRUIT:
+        {
+            string taste = getMiscString("eating_fruit");
+            if (taste.empty())
+                taste = "Eugh, buggy fruit.";
+            mpr(taste);
+            break;
+        }
+        default:
+            break;
+        }
+    }
+
+    switch (type)
+    {
+    case FOOD_ROYAL_JELLY:
+        mpr("That royal jelly was delicious!");
+        break;
+    case FOOD_PIZZA:
+    {
+        if (!Options.pizzas.empty())
+        {
+            const string za = Options.pizzas[random2(Options.pizzas.size())];
+            mprf("Mmm... %s.", trimmed_string(za).c_str());
+            break;
+        }
+
+        const string taste = getMiscString("eating_pizza");
+        if (taste.empty())
+        {
+            mpr("Bleh, bug pizza.");
+            break;
+        }
+
+        mprf("%s", taste.c_str());
+        break;
+    }
+    default:
+        break;
+    }
+}
+
+
+void finish_eating_item(item_def& food)
+{
+    if (food.sub_type == FOOD_CHUNK)
         _eat_chunk(food);
     else
-        _eating(food);
-
-    you.turn_is_over = true;
+    {
+        int value = food_value(food);
+        ASSERT(value > 0);
+        lessen_hunger(value, true);
+        _finished_eating_message(static_cast<food_type>(food.sub_type));
+    }
 
     count_action(CACT_EAT, food.sub_type);
+
     if (is_perishable_stack(food)) // chunks
         remove_oldest_perishable_item(food);
     if (in_inventory(food))
-        dec_inv_item_quantity(link, 1);
+        dec_inv_item_quantity(food.link, 1);
     else
-        dec_mitm_item_quantity(link, 1);
-
-    return true;
+        dec_mitm_item_quantity(food.index(), 1);
 }
 
 // Returns which of two food items is older (true for first, else false).
@@ -494,249 +547,11 @@ static bool _compare_by_freshness(const item_def *food1, const item_def *food2)
     return food1->freshness < food2->freshness;
 }
 
-#ifdef TOUCH_UI
-static string _floor_eat_menu_title(const Menu *menu, const string &oldt)
-{
-    return oldt;
-}
-#endif
-
-// Returns -1 for cancel, 1 for eaten, 0 for not eaten.
-int eat_from_floor(bool skip_chunks)
-{
-    if (!_eat_check())
-        return false;
-
-    // Corpses should have been handled before.
-    if (you.species == SP_VAMPIRE)
-        return 0;
-
-    bool need_more = false;
-    int inedible_food = 0;
-    item_def wonteat;
-    bool found_valid = false;
-
-#ifdef TOUCH_UI
-    vector<const item_def*> food_items;
-#else
-    vector<item_def*> food_items;
-#endif
-    for (stack_iterator si(you.pos(), true); si; ++si)
-    {
-        if (si->base_type != OBJ_FOOD)
-            continue;
-
-        // Chunks should have been handled before.
-        if (skip_chunks && si->sub_type == FOOD_CHUNK)
-            continue;
-
-        if (is_bad_food(*si))
-            continue;
-
-        if (!can_eat(*si, true))
-        {
-            if (!inedible_food)
-            {
-                wonteat = *si;
-                inedible_food++;
-            }
-            else
-            {
-                // Increase only if we're dealing with different subtypes.
-                // FIXME: Use a common check for herbivorous/carnivorous
-                //        dislikes, for e.g. "Blech! You need blood!"
-                ASSERT(wonteat.defined());
-                if (wonteat.sub_type != si->sub_type)
-                    inedible_food++;
-            }
-
-            continue;
-        }
-
-        found_valid = true;
-        food_items.push_back(&(*si));
-    }
-
-    if (found_valid)
-    {
-#ifdef TOUCH_UI
-        redraw_screen();
-        for (SelItem &sel : select_items(food_items, "Eat", false, MT_SELONE,
-                                         _floor_eat_menu_title))
-        {
-            item_def *item = const_cast<item_def *>(sel.item);
-            if (!check_warning_inscriptions(*item, OPER_EAT))
-                break;
-
-            if (can_eat(*item, false))
-                return eat_item(*item);
-        }
-#else
-        sort(food_items.begin(), food_items.end(), _compare_by_freshness);
-        for (item_def *item : food_items)
-        {
-            string item_name = get_menu_colour_prefix_tags(*item, DESC_A);
-
-            mprf(MSGCH_PROMPT, "%s %s%s? (ye/n/q/i?)",
-                 "Eat",
-                 ((item->quantity > 1) ? "one of " : ""),
-                 item_name.c_str());
-
-            int keyin = toalower(getchm(KMC_CONFIRM));
-            switch (keyin)
-            {
-            case 'q':
-            CASE_ESCAPE
-                canned_msg(MSG_OK);
-                return -1;
-            case 'e':
-            case 'y':
-                if (!check_warning_inscriptions(*item, OPER_EAT))
-                    break;
-
-                if (can_eat(*item, false))
-                    return eat_item(*item);
-                need_more = true;
-                break;
-            case 'i':
-            case '?':
-                // Directly skip ahead to inventory.
-                return 0;
-            default:
-                // Else no: try next one.
-                break;
-            }
-        }
-#endif
-    }
-    else if (inedible_food)
-    {
-        if (inedible_food == 1)
-        {
-            ASSERT(wonteat.defined());
-            // Use the normal cannot ingest message.
-            if (can_eat(wonteat, false))
-            {
-                mprf(MSGCH_DIAGNOSTICS, "Error: Can eat %s after all?",
-                     wonteat.name(DESC_PLAIN).c_str());
-            }
-        }
-        else // Several different food items.
-            mpr("You refuse to eat these food items.");
-        need_more = true;
-    }
-
-    if (need_more)
-        more();
-
-    return 0;
-}
-
-bool eat_from_inventory()
-{
-    if (!_eat_check())
-        return false;
-
-    // Corpses should have been handled before.
-    if (you.species == SP_VAMPIRE)
-        return 0;
-
-    int inedible_food = 0;
-    item_def *wonteat = nullptr;
-    bool found_valid = false;
-
-    vector<item_def *> food_items;
-    for (auto &item : you.inv)
-    {
-        if (!item.defined())
-            continue;
-
-        // Chunks should have been handled before.
-        if (item.base_type != OBJ_FOOD || item.sub_type == FOOD_CHUNK)
-            continue;
-
-        if (is_bad_food(item))
-            continue;
-
-        if (!can_eat(item, true))
-        {
-            if (!inedible_food)
-            {
-                wonteat = &item;
-                inedible_food++;
-            }
-            else
-            {
-                // Increase only if we're dealing with different subtypes.
-                // FIXME: Use a common check for herbivorous/carnivorous
-                //        dislikes, for e.g. "Blech! You need blood!"
-                ASSERT(wonteat->defined());
-                if (wonteat->sub_type != item.sub_type)
-                    inedible_food++;
-            }
-            continue;
-        }
-
-        found_valid = true;
-        food_items.push_back(&item);
-    }
-
-    if (found_valid)
-    {
-        sort(food_items.begin(), food_items.end(), _compare_by_freshness);
-        for (item_def *item : food_items)
-        {
-            string item_name = get_menu_colour_prefix_tags(*item, DESC_A);
-
-            mprf(MSGCH_PROMPT, "%s %s%s? (ye/n/q)",
-                 "Eat",
-                 ((item->quantity > 1) ? "one of " : ""),
-                 item_name.c_str());
-
-            int keyin = toalower(getchm(KMC_CONFIRM));
-            switch (keyin)
-            {
-            case 'q':
-            CASE_ESCAPE
-                canned_msg(MSG_OK);
-                return false;
-            case 'e':
-            case 'y':
-                if (can_eat(*item, false))
-                    return eat_item(*item);
-                break;
-            default:
-                // Else no: try next one.
-                break;
-            }
-        }
-    }
-    else if (inedible_food)
-    {
-        if (inedible_food == 1)
-        {
-            ASSERT(wonteat->defined());
-            // Use the normal cannot ingest message.
-            if (can_eat(*wonteat, false))
-            {
-                mprf(MSGCH_DIAGNOSTICS, "Error: Can eat %s after all?",
-                    wonteat->name(DESC_PLAIN).c_str());
-            }
-        }
-        else // Several different food items.
-            mpr("You refuse to eat these food items.");
-    }
-
-    return false;
-}
-
-
 /** Make the prompt for chunk eating/corpse draining.
  *
  *  @param only_auto Don't actually make a prompt: if there are
  *                   things to auto_eat, eat them, and exit otherwise.
  *  @returns -1 for cancel, 1 for eaten, 0 for not eaten,
- *           -2 for skip to inventory.
  */
 int prompt_eat_chunks(bool only_auto)
 {
@@ -805,7 +620,7 @@ int prompt_eat_chunks(bool only_auto)
         for (item_def *item : chunks)
         {
             bool autoeat = false;
-            string item_name = get_menu_colour_prefix_tags(*item, DESC_A);
+            string item_name = menu_colour_item_name(*item, DESC_A);
 
             const bool bad = is_bad_food(*item);
 
@@ -822,7 +637,7 @@ int prompt_eat_chunks(bool only_auto)
                 return 0;
             else
             {
-                mprf(MSGCH_PROMPT, "%s %s%s? (ye/n/q/i?)",
+                mprf(MSGCH_PROMPT, "%s %s%s? (ye/n/q)",
                      (you.species == SP_VAMPIRE ? "Drink blood from" : "Eat"),
                      ((item->quantity > 1) ? "one of " : ""),
                      item_name.c_str());
@@ -838,7 +653,7 @@ int prompt_eat_chunks(bool only_auto)
             case 'i':
             case '?':
                 // Skip ahead to the inventory.
-                return -2;
+                return 0;
             case 'e':
             case 'y':
                 if (can_eat(*item, false))
@@ -892,7 +707,7 @@ static const char *_chunk_flavour_phrase(bool likes_chunks)
     return phrase;
 }
 
-void chunk_nutrition_message(int nutrition)
+static void _chunk_nutrition_message(int nutrition)
 {
     int perc_nutrition = nutrition * 100 / CHUNK_BASE_NUTRITION;
     if (perc_nutrition < 15)
@@ -1004,111 +819,9 @@ static void _eat_chunk(item_def& food)
     if (do_eat)
     {
         dprf("nutrition: %d", nutrition);
-        start_delay(DELAY_EAT, food_turns(food) - 1,
-                    (suppress_msg) ? 0 : nutrition, -1);
         lessen_hunger(nutrition, true);
-    }
-}
-
-static void _eating(item_def& food)
-{
-    int food_value = ::food_value(food);
-    ASSERT(food_value > 0);
-
-    int duration = food_turns(food) - 1;
-
-    // use delay.parm3 to figure out whether to output "finish eating"
-    start_delay(DELAY_EAT, duration, 0, food.sub_type, duration);
-
-    lessen_hunger(food_value, true);
-}
-
-// Handle messaging at the end of eating.
-// Some food types may not get a message.
-void finished_eating_message(int food_type)
-{
-    bool herbivorous = player_mutation_level(MUT_HERBIVOROUS) > 0;
-    bool carnivorous = player_mutation_level(MUT_CARNIVOROUS) > 0;
-
-    if (herbivorous)
-    {
-        if (food_is_meaty(food_type))
-        {
-            mpr("Blech - you need greens!");
-            return;
-        }
-    }
-    else
-    {
-        switch (food_type)
-        {
-        case FOOD_MEAT_RATION:
-            mpr("That meat ration really hit the spot!");
-            return;
-        case FOOD_BEEF_JERKY:
-            mprf("That beef jerky was %s!",
-                 one_chance_in(4) ? "jerk-a-riffic"
-                                  : "delicious");
-            return;
-        default:
-            break;
-        }
-    }
-
-    if (carnivorous)
-    {
-        if (food_is_veggie(food_type))
-        {
-            mpr("Blech - you need meat!");
-            return;
-        }
-    }
-    else
-    {
-        switch (food_type)
-        {
-        case FOOD_BREAD_RATION:
-            mpr("That bread ration really hit the spot!");
-            return;
-        case FOOD_FRUIT:
-        {
-            string taste = getMiscString("eating_fruit");
-            if (taste.empty())
-                taste = "Eugh, buggy fruit.";
-            mpr(taste);
-            break;
-        }
-        default:
-            break;
-        }
-    }
-
-    switch (food_type)
-    {
-    case FOOD_ROYAL_JELLY:
-        mpr("That royal jelly was delicious!");
-        break;
-    case FOOD_PIZZA:
-    {
-        if (!Options.pizzas.empty())
-        {
-            const string za = Options.pizzas[random2(Options.pizzas.size())];
-            mprf("Mmm... %s.", trimmed_string(za).c_str());
-            break;
-        }
-
-        const string taste = getMiscString("eating_pizza");
-        if (taste.empty())
-        {
-            mpr("Bleh, bug pizza.");
-            break;
-        }
-
-        mprf("%s", taste.c_str());
-        break;
-    }
-    default:
-        break;
+        if (!suppress_msg)
+            _chunk_nutrition_message(nutrition);
     }
 }
 
@@ -1396,13 +1109,9 @@ corpse_effect_type determine_chunk_effect(corpse_effect_type chunktype)
     return chunktype;
 }
 
-static bool _vampire_consume_corpse(int slot, bool invent)
+static bool _vampire_consume_corpse(item_def& corpse)
 {
     ASSERT(you.species == SP_VAMPIRE);
-
-    item_def &corpse = (invent ? you.inv[slot]
-                               : mitm[slot]);
-
     ASSERT(corpse.base_type == OBJ_CORPSES);
     ASSERT(corpse.sub_type == CORPSE_BODY);
 
@@ -1425,7 +1134,7 @@ static bool _vampire_consume_corpse(int slot, bool invent)
 
     // The draining delay doesn't have a start action, and we only need
     // the continue/finish messages if it takes longer than 1 turn.
-    start_delay(DELAY_FEED_VAMPIRE, duration, invent, slot);
+    start_delay<FeedVampireDelay>(duration, corpse);
 
     return true;
 }
@@ -1473,7 +1182,7 @@ int you_min_hunger()
 void handle_starvation()
 {
     // Don't faint or die while eating.
-    if (current_delay_action() == DELAY_EAT)
+    if (current_delay() && current_delay()->is_being_used(nullptr, OPER_EAT))
         return;
 
     if (!you_foodless() && you.hunger <= HUNGER_FAINTING)
@@ -1511,7 +1220,7 @@ void handle_starvation()
 
             mprf(MSGCH_FOOD, "You have starved to death.");
             ouch(INSTANT_DEATH, KILLED_BY_STARVATION);
-            if (!you.dead) // if we're still here...
+            if (!you.pending_revival) // if we're still here...
                 set_hunger(HUNGER_DEFAULT, true);
         }
     }
