@@ -24,6 +24,7 @@
 #define RU_SACRIFICE_PROGRESS_KEY "ru_progress_to_next_sacrifice"
 #define RU_SACRIFICE_DELAY_KEY "ru_sacrifice_delay"
 #define RU_SACRIFICE_PENALTY_KEY "ru_sacrifice_penalty"
+#define RU_SAC_XP_LEVELS 2
 
 const char * const GOZAG_POTIONS_KEY = "gozag_potions%d";
 const char * const GOZAG_PRICE_KEY = "gozag_price%d";
@@ -41,16 +42,25 @@ const char * const GOZAG_SHOP_COST_KEY       = "gozag_shop_cost_%d";
 #define GOZAG_MAX_BRIBABILITY 8
 #define GOZAG_MAX_POTIONS 3
 
-#define RU_SAC_XP_LEVELS 2
+#define USKAYAW_AUDIENCE_TIMER "uskayaw_audience_timer"
+#define USKAYAW_BOND_TIMER "uskayaw_bond_timer"
+#define USKAYAW_NUM_MONSTERS_HURT "uskayaw_num_monsters_hurt"
+#define USKAYAW_MONSTER_HURT_VALUE "uskayaw_monster_hurt_value"
+#define USKAYAW_AUT_SINCE_PIETY_GAIN "uskayaw_aut_since_piety_gain"
 
 struct bolt;
 class stack_iterator;
 
+typedef FixedVector<int, NUM_RECITE_TYPES> recite_counts;
+
 bool can_do_capstone_ability(god_type god);
 bool bless_weapon(god_type god, brand_type brand, colour_t colour);
 bool zin_donate_gold();
 string zin_recite_text(const int seed, const int prayertype, int step);
 bool zin_check_able_to_recite(bool quiet = false);
+recite_eligibility zin_check_recite_to_single_monster(const monster *mon,
+                                                  recite_counts &eligibility,
+                                                  bool quiet = false);
 int zin_check_recite_to_monsters(bool quiet = false);
 bool zin_recite_to_single_monster(const coord_def& where);
 int zin_recite_power();
@@ -68,11 +78,12 @@ void elyvilon_remove_divine_vigour();
 
 bool vehumet_supports_spell(spell_type spell);
 
+void sif_do_channel_energy(int pow);
+
 bool trog_burn_spellbooks();
 void trog_do_trogs_hand(int power);
 void trog_remove_trogs_hand();
 
-void jiyva_paralyse_jellies();
 bool jiyva_remove_bad_mutation();
 
 bool beogh_can_gift_items_to(const monster* mons, bool quiet = true);
@@ -150,17 +161,29 @@ void init_sac_index();
 int get_sacrifice_piety(ability_type sac, bool include_skill = true);
 void ru_offer_new_sacrifices();
 string ru_sac_text(ability_type sac);
+string ru_sacrifice_description(ability_type sac);
 bool ru_do_sacrifice(ability_type sac);
-bool ru_reject_sacrifices(bool skip_prompt = false);
-void ru_reset_sacrifice_timer(bool clear_timer = false);
+bool ru_reject_sacrifices(bool forced_rejection = false);
+void ru_reset_sacrifice_timer(bool clear_timer = false,
+                              bool faith_penalty = false);
 bool will_ru_retaliate();
 void ru_do_retribution(monster* mons, int damage);
 void ru_draw_out_power();
 bool ru_power_leap();
+int cell_has_valid_target(coord_def where);
 bool ru_apocalypse();
 string ru_sacrifice_vector(ability_type sac);
 
 bool pakellas_check_quick_charge(bool quiet);
 int pakellas_effective_hex_power(int pow);
-bool pakellas_device_surge();
+int pakellas_surge_devices();
+
+bool uskayaw_stomp();
+bool uskayaw_line_pass();
+bool uskayaw_grand_finale();
+
+bool hepliaklqana_choose_ancestor_type(int ancestor_type);
+spret_type hepliaklqana_idealise(bool fail);
+spret_type hepliaklqana_transference(bool fail);
+void hepliaklqana_choose_identity();
 #endif
