@@ -16,6 +16,7 @@
 #include "files.h"
 #include "food.h"
 #include "fprop.h"
+#include "godabil.h"
 #include "goditem.h"
 #include "godprayer.h"
 #include "invent.h" // in_inventory
@@ -26,6 +27,7 @@
 #include "message.h"
 #include "mon-cast.h"
 #include "mon-place.h"
+#include "mon-util.h"
 #include "religion.h"
 #include "shout.h"
 #include "skills.h"
@@ -88,18 +90,16 @@ static const vector<god_passive> god_passives[NUM_GODS] =
         { -1, passive_t::protect_from_harm, "GOD sometimes watches over you" },
         { -1, passive_t::abjuration_protection_hd,
               "GOD protects your summons from abjuration" },
-        { -1, passive_t::bless_followers_vs_unholy,
-              "GOD blesses your followers when they kill evil or unholy beings"
+        { -1, passive_t::bless_followers_vs_evil,
+              "GOD blesses your followers when they kill evil beings"
         },
-        { -1, passive_t::restore_hp_mp_vs_unholy,
-              "gain health and magic from killing evil or unholy beings" },
+        { -1, passive_t::restore_hp_mp_vs_evil,
+              "gain health and magic from killing evil beings" },
         {  0, passive_t::halo, "are surrounded by divine halo" },
     },
 
     // Kikubaaqudgha
     {
-        {  0, passive_t::deaths_door_hp_boost,
-              "can retain more of your health when standing in Death's Door" },
         {  2, passive_t::miscast_protection_necromancy,
               "GOD protects you from necromancy miscasts and mummy death curses"
         },
@@ -170,7 +170,6 @@ static const vector<god_passive> god_passives[NUM_GODS] =
     // Beogh
     {
         { -1, passive_t::share_exp, "share experience with your followers" },
-        {  0, passive_t::bonus_ac, "gain increased benefits from armour" },
         {  3, passive_t::convert_orcs, "inspire orcs to join your side" },
         {  3, passive_t::bless_followers,
               "GOD will bless your followers.",
@@ -183,7 +182,7 @@ static const vector<god_passive> god_passives[NUM_GODS] =
     {
         { -1, passive_t::neutral_slimes, "slimes and eye monsters are neutral towards you" },
         { -1, passive_t::jellies_army, "GOD summons jellies to protect you" },
-        { -1, passive_t::jelly_eating, "GOD allows jellies to devour more items" },
+        { -1, passive_t::jelly_eating, "GOD allows jellies to devour items" },
         { -1, passive_t::fluid_stats, "GOD adjusts your attributes periodically" },
         {  2, passive_t::slime_feed, "items consumed by your fellow slimes feed you" },
         {  3, passive_t::resist_corrosion, "GOD protects your from corrosion" },
@@ -259,7 +258,7 @@ static const vector<god_passive> god_passives[NUM_GODS] =
 
     // Qazlal
     {
-        {  0, passive_t::resist_own_clouds, "clouds generated by your actions don't harm you" },
+        {  0, passive_t::cloud_immunity, "clouds can't harm you" },
         {  1, passive_t::storm_shield, "generate elemental clouds to protect you" },
         {  4, passive_t::upgraded_storm_shield, "chances to be struck by projectiles are reduced" },
         {  5, passive_t::elemental_adaptation, "elemental attacks leaves you somewhat more resistant to themxo" }
@@ -275,6 +274,17 @@ static const vector<god_passive> god_passives[NUM_GODS] =
     {
         { -1, passive_t::no_mp_regen, "GOD prevents you from regenerating your mana reserve" },
         { -1, passive_t::mp_on_kill, "have a chance to gain mana when you kill" },
+        {  0, passive_t::identify_devices, "GOD identifies your wands and rods" },
+        {  1, passive_t::bottle_mp, "GOD collects and distills excess magic from your kills" },
+    },
+
+    // Uskayaw
+    { },
+
+    // Hepliaklqana
+    {
+        { -1, passive_t::frail, "GOD siphons a part of your essence into your ancestor" },
+        {  5, passive_t::transfer_drain, "drain nearby creatures when transferring your ancestor" },
     },
 };
 
@@ -411,7 +421,7 @@ void ash_check_bondage(bool msg)
 
     int cursed[NUM_ET] = {0}, slots[NUM_ET] = {0};
 
-    for (int j = EQ_WEAPON; j < NUM_EQUIP; j++)
+    for (int j = EQ_FIRST_EQUIP; j < NUM_EQUIP; j++)
     {
         const equipment_type i = static_cast<equipment_type>(j);
         eq_type s;
@@ -581,10 +591,11 @@ string ash_describe_bondage(int flags, bool level)
         && you.bondage[ET_ARMOUR] == you.bondage[ET_JEWELS]
         && you.bondage[ET_ARMOUR] != -1)
     {
-        desc += make_stringf("You are %s bound in armour and magic.\n",
+        desc += make_stringf("You are %s bound in armour %s jewellery.\n",
                              you.bondage[ET_ARMOUR] == 0 ? "not" :
                              you.bondage[ET_ARMOUR] == 1 ? "partially"
-                                                         : "fully");
+                                                         : "fully",
+                             you.bondage[ET_ARMOUR] == 0 ? "or" : "and");
     }
     else
     {
@@ -598,7 +609,7 @@ string ash_describe_bondage(int flags, bool level)
 
         if (flags & ETF_JEWELS && you.bondage[ET_JEWELS] != -1)
         {
-            desc += make_stringf("You are %s bound in magic.\n",
+            desc += make_stringf("You are %s bound in jewellery.\n",
                                  you.bondage[ET_JEWELS] == 0 ? "not" :
                                  you.bondage[ET_JEWELS] == 1 ? "partially"
                                                              : "fully");
@@ -654,8 +665,11 @@ bool god_id_item(item_def& item, bool silent)
             item.props["needs_autopickup"] = true;
         }
 
-        if (is_weapon(item) || item.base_type == OBJ_ARMOUR)
+        if (is_weapon(item) || item.base_type == OBJ_RODS
+            || item.base_type == OBJ_ARMOUR)
+        {
             ided |= ISFLAG_KNOW_PROPERTIES | ISFLAG_KNOW_TYPE;
+        }
 
         if (item.base_type == OBJ_JEWELLERY)
             ided |= ISFLAG_IDENT_MASK;
@@ -707,31 +721,26 @@ void ash_id_monster_equipment(monster* mon)
 
     for (unsigned int i = 0; i <= MSLOT_LAST_VISIBLE_SLOT; ++i)
     {
-        // Wielded weapon brands are IDed for everyone already.
-        if (i == MSLOT_WEAPON)
-            continue;
         if (mon->inv[i] == NON_ITEM)
             continue;
 
         item_def &item = mitm[mon->inv[i]];
         if ((i != MSLOT_WAND || !is_offensive_wand(item))
-            && !item_is_branded(item))
+            && !item_is_branded(item)
+            && item.base_type != OBJ_RODS)
         {
             continue;
         }
 
-        if (x_chance_in_y(piety_rank(), 6))
+        if (i == MSLOT_WAND)
         {
-            if (i == MSLOT_WAND)
-            {
-                set_ident_type(OBJ_WANDS, item.sub_type, true);
-                mon->props["wand_known"] = true;
-            }
-            else
-                set_ident_flags(item, ISFLAG_KNOW_TYPE);
-
-            id = true;
+            set_ident_type(OBJ_WANDS, item.sub_type, true);
+            mon->props["wand_known"] = true;
         }
+        else
+            set_ident_flags(item, ISFLAG_KNOW_TYPE);
+
+        id = true;
     }
 
     if (id)
@@ -998,7 +1007,7 @@ void qazlal_storm_clouds()
         do
         {
             ctype = random_choose(CLOUD_FIRE, CLOUD_COLD, CLOUD_STORM,
-                                       CLOUD_DUST_TRAIL);
+                                  CLOUD_DUST);
         } while (water && ctype == CLOUD_FIRE);
 
         place_cloud(ctype, candidates[i], random_range(3, 5), &you);
@@ -1105,7 +1114,7 @@ bool does_ru_wanna_redirect(monster* mon)
     return have_passive(passive_t::aura_of_power)
             && !mon->friendly()
             && you.see_cell(mon->pos())
-            && !mons_is_firewood(mon)
+            && !mons_is_firewood(*mon)
             && !mon->submerged()
             && !mons_is_projectile(mon->type);
 }
@@ -1151,7 +1160,7 @@ void pakellas_id_device_charges()
             set_ident_flags(you.inv[which_item], ISFLAG_KNOW_TYPE);
         set_ident_flags(you.inv[which_item], ISFLAG_KNOW_PLUSES);
         mprf_nocap("%s",
-                   get_menu_colour_prefix_tags(you.inv[which_item],
+                   menu_colour_item_name(you.inv[which_item],
                                                DESC_INVENTORY).c_str());
     }
 }
@@ -1364,3 +1373,97 @@ void dithmenos_shadow_spell(bolt* orig_beam, spell_type spell)
 
     shadow_monster_reset(mon);
 }
+
+/**
+ * check if the monster in this cell exists and is a valid target for Uskayaw
+ */
+static int _check_for_uskayaw_targets(coord_def where)
+{
+    if (!cell_has_valid_target(where))
+        return 0;
+    monster* mons = monster_at(where);
+    ASSERT(mons);
+
+    if (mons_is_firewood(*mons))
+        return 0;
+
+    return 1;
+}
+
+/**
+ * Paralyze the monster in this cell, assuming one exists.
+ *
+ * Duration increases with invocations and experience level, and decreases
+ * with target HD. The duration is pretty low, maxing out at 40 AUT.
+ */
+static int _prepare_audience(coord_def where)
+{
+    if (!_check_for_uskayaw_targets(where))
+        return 0;
+
+    monster* mons = monster_at(where);
+
+    int power =  max(1, random2(1 + you.skill(SK_INVOCATIONS, 2))
+                 + you.experience_level - mons->get_hit_dice());
+    int duration = min(max(10, 5 + power), 40);
+    mons->add_ench(mon_enchant(ENCH_PARALYSIS, 1, &you, duration));
+
+    return 1;
+}
+
+/**
+ * On hitting *** piety, all the monsters are paralysed by their appreciation
+ * for your dance.
+ */
+void uskayaw_prepares_audience()
+{
+    int count = apply_area_visible(_check_for_uskayaw_targets, you.pos());
+    if (count > 0)
+    {
+        simple_god_message(" prepares the audience for your solo!");
+        apply_area_visible(_prepare_audience, you.pos());
+
+        // Increment a delay timer to prevent players from spamming this ability
+        // via piety loss and gain. Timer is in AUT.
+        you.props[USKAYAW_AUDIENCE_TIMER] = 300 + random2(201);
+    }
+    else // Reset the timer because we didn't actually execute.
+        you.props[USKAYAW_AUDIENCE_TIMER] = 0;
+}
+
+/**
+ * Apply pain bond to the monster in this cell.
+ */
+static int _bond_audience(coord_def where)
+{
+    if (!_check_for_uskayaw_targets(where))
+        return 0;
+
+    monster* mons = monster_at(where);
+
+    int power = you.skill(SK_INVOCATIONS, 7) + you.experience_level
+                 - mons->get_hit_dice();
+    int duration = 20 + random2avg(power, 2);
+    mons->add_ench(mon_enchant(ENCH_PAIN_BOND, 1, &you, duration));
+
+    return 1;
+}
+
+/**
+ * On hitting **** piety, all the monsters are pain bonded.
+ */
+void uskayaw_bonds_audience()
+{
+    int count = apply_area_visible(_check_for_uskayaw_targets, you.pos());
+    if (count > 1)
+    {
+        simple_god_message(" links your audience in an emotional bond!");
+        apply_area_visible(_bond_audience, you.pos());
+
+        // Increment a delay timer to prevent players from spamming this ability
+        // via piety loss and gain. Timer is in AUT.
+        you.props[USKAYAW_BOND_TIMER] = 300 + random2(201);
+    }
+    else // Reset the timer because we didn't actually execute.
+        you.props[USKAYAW_BOND_TIMER] = 0;
+}
