@@ -47,6 +47,7 @@
 #include "terrain.h"
 #include "timed_effects.h"
 #include "traps.h"
+#include "unwind.h"
 #include "view.h"
 #include "xom.h"
 
@@ -127,7 +128,7 @@ bool monster::add_ench(const mon_enchant &ench)
         return false;
 
     if (ench.ench == ENCH_FEAR
-        && (holiness() & MH_NONLIVING || berserk_or_insane()))
+        && (is_nonliving() || berserk_or_insane()))
     {
         return false;
     }
@@ -239,6 +240,15 @@ void monster::add_enchantment_effect(const mon_enchant &ench, bool quiet)
                 target = menv[source_actor->as_monster()->foe].pos();
         }
 
+        if (type == MONS_FLAYED_GHOST)
+        {
+            // temporarly change our attitude back (XXX: scary code...)
+            unwind_var<mon_enchant_list> enchants(enchantments, {});
+            unwind_var<FixedBitVector<NUM_ENCHANTMENTS>> ecache(ench_cache, {});
+            end_flayed_effect(this);
+        }
+        del_ench(ENCH_STILL_WINDS);
+
         if (is_patrolling())
         {
             // Enslaved monsters stop patrolling and forget their patrol
@@ -280,10 +290,6 @@ void monster::add_enchantment_effect(const mon_enchant &ench, bool quiet)
         invalidate_agrid(true);
         break;
 
-    case ENCH_ROLLING:
-        calc_speed();
-        break;
-
     case ENCH_FROZEN:
         calc_speed();
         break;
@@ -296,6 +302,10 @@ void monster::add_enchantment_effect(const mon_enchant &ench, bool quiet)
         }
         break;
 
+    case ENCH_STILL_WINDS:
+        start_still_winds();
+        break;
+
     default:
         break;
     }
@@ -428,16 +438,16 @@ void monster::remove_enchantment_effect(const mon_enchant &me, bool quiet)
     case ENCH_HASTE:
         calc_speed();
         if (!quiet)
-            simple_monster_message(this, " is no longer moving quickly.");
+            simple_monster_message(*this, " is no longer moving quickly.");
         break;
 
     case ENCH_SWIFT:
         if (!quiet)
         {
             if (type == MONS_ALLIGATOR)
-                simple_monster_message(this, " slows down.");
+                simple_monster_message(*this, " slows down.");
             else
-                simple_monster_message(this, " is no longer moving somewhat quickly.");
+                simple_monster_message(*this, " is no longer moving somewhat quickly.");
         }
         break;
 
@@ -446,7 +456,7 @@ void monster::remove_enchantment_effect(const mon_enchant &me, bool quiet)
         if (!quiet && !silenced(pos()))
         {
             if (alive())
-                simple_monster_message(this, " becomes audible again.");
+                simple_monster_message(*this, " becomes audible again.");
             else
                 mprf("As %s %s, the sound returns.",
                      name(DESC_THE).c_str(),
@@ -456,23 +466,15 @@ void monster::remove_enchantment_effect(const mon_enchant &me, bool quiet)
 
     case ENCH_MIGHT:
         if (!quiet)
-            simple_monster_message(this, " no longer looks unusually strong.");
+            simple_monster_message(*this, " no longer looks unusually strong.");
         break;
 
     case ENCH_SLOW:
         if (!quiet)
-            simple_monster_message(this, " is no longer moving slowly.");
+            simple_monster_message(*this, " is no longer moving slowly.");
         calc_speed();
         break;
 
-    case ENCH_MAGIC_ARMOUR:
-        if (!quiet && you.can_see(*this))
-        {
-            mprf("%s magical armour fades away.",
-                 apostrophise(name(DESC_THE)).c_str());
-        }
-        break;
-
     case ENCH_OZOCUBUS_ARMOUR:
         if (!quiet && you.can_see(*this))
         {
@@ -483,14 +485,14 @@ void monster::remove_enchantment_effect(const mon_enchant &me, bool quiet)
 
     case ENCH_PARALYSIS:
         if (!quiet)
-            simple_monster_message(this, " is no longer paralysed.");
+            simple_monster_message(*this, " is no longer paralysed.");
 
         behaviour_event(this, ME_EVAL);
         break;
 
     case ENCH_PETRIFIED:
         if (!quiet)
-            simple_monster_message(this, " is no longer petrified.");
+            simple_monster_message(*this, " is no longer petrified.");
         del_ench(ENCH_PETRIFYING);
 
         behaviour_event(this, ME_EVAL);
@@ -506,7 +508,7 @@ void monster::remove_enchantment_effect(const mon_enchant &me, bool quiet)
     case ENCH_FEAR:
     {
         string msg;
-        if (holiness() & MH_NONLIVING || berserk_or_insane())
+        if (is_nonliving() || berserk_or_insane())
         {
             // This should only happen because of fleeing sanctuary
             msg = " stops retreating.";
@@ -518,7 +520,7 @@ void monster::remove_enchantment_effect(const mon_enchant &me, bool quiet)
         }
 
         if (!quiet)
-            simple_monster_message(this, msg.c_str());
+            simple_monster_message(*this, msg.c_str());
 
         // Reevaluate behaviour.
         behaviour_event(this, ME_EVAL);
@@ -527,7 +529,7 @@ void monster::remove_enchantment_effect(const mon_enchant &me, bool quiet)
 
     case ENCH_CONFUSION:
         if (!quiet)
-            simple_monster_message(this, " seems less confused.");
+            simple_monster_message(*this, " seems less confused.");
 
         // Reevaluate behaviour.
         behaviour_event(this, ME_EVAL);
@@ -539,7 +541,7 @@ void monster::remove_enchantment_effect(const mon_enchant &me, bool quiet)
         // before being polymorphed into a non-invisible monster.
         if (you.see_cell(pos()) && !you.can_see_invisible() && !backlit()
             && !has_ench(ENCH_SUBMERGED)
-            && !friendly() && !you.duration[DUR_TELEPATHY])
+            && !friendly())
         {
             if (!quiet)
                 mprf("%s appears from thin air!", name(DESC_A, true).c_str());
@@ -582,11 +584,11 @@ void monster::remove_enchantment_effect(const mon_enchant &me, bool quiet)
             {
                 if (me.ench == ENCH_CHARM && props.exists("charmed_demon"))
                 {
-                    simple_monster_message(this,
+                    simple_monster_message(*this,
                                            " breaks free of your control!");
                 }
                 else
-                    simple_monster_message(this,
+                    simple_monster_message(*this,
                                         me.ench == ENCH_CHARM
                                         ? " is no longer charmed."
                                         : me.ench == ENCH_HEXED
@@ -627,7 +629,7 @@ void monster::remove_enchantment_effect(const mon_enchant &me, bool quiet)
     if (!quiet)
         {
             if (visible_to(&you))
-                simple_monster_message(this, " stops glowing.");
+                simple_monster_message(*this, " stops glowing.");
             else if (has_ench(ENCH_INVIS) && you.see_cell(pos()))
             {
                 mprf("%s stops glowing and disappears.",
@@ -638,12 +640,12 @@ void monster::remove_enchantment_effect(const mon_enchant &me, bool quiet)
 
     case ENCH_STICKY_FLAME:
         if (!quiet)
-            simple_monster_message(this, " stops burning.");
+            simple_monster_message(*this, " stops burning.");
         break;
 
     case ENCH_POISON:
         if (!quiet)
-            simple_monster_message(this, " looks more healthy.");
+            simple_monster_message(*this, " looks more healthy.");
         break;
 
     case ENCH_HELD:
@@ -657,7 +659,7 @@ void monster::remove_enchantment_effect(const mon_enchant &me, bool quiet)
                 props.erase(NEWLY_TRAPPED_KEY);
 
             if (!quiet)
-                simple_monster_message(this, " breaks free.");
+                simple_monster_message(*this, " breaks free.");
         }
         break;
     }
@@ -675,7 +677,7 @@ void monster::remove_enchantment_effect(const mon_enchant &me, bool quiet)
                 ENCH_ABJ : ENCH_FAKE_ABJURATION, 0, 0, -1));
 
         if (berserk())
-            simple_monster_message(this, " is no longer berserk.");
+            simple_monster_message(*this, " is no longer berserk.");
 
         monster_die(this, (me.ench == ENCH_FAKE_ABJURATION) ? KILL_MISC :
                             (quiet) ? KILL_DISMISSED : KILL_RESET, NON_MONSTER);
@@ -686,7 +688,7 @@ void monster::remove_enchantment_effect(const mon_enchant &me, bool quiet)
         monster_die(this, KILL_TIMEOUT, NON_MONSTER);
         break;
     case ENCH_SUBMERGED:
-        if (mons_is_wandering(this))
+        if (mons_is_wandering(*this))
         {
             behaviour = BEH_SEEK;
             behaviour_event(this, ME_EVAL);
@@ -723,7 +725,7 @@ void monster::remove_enchantment_effect(const mon_enchant &me, bool quiet)
     case ENCH_SOUL_RIPE:
         if (!quiet)
         {
-            simple_monster_message(this,
+            simple_monster_message(*this,
                                    "'s soul is no longer ripe for the taking.");
         }
         break;
@@ -734,16 +736,11 @@ void monster::remove_enchantment_effect(const mon_enchant &me, bool quiet)
             forest_message(pos(), "The forest calms down.");
         break;
 
-    case ENCH_WITHDRAWN:
-        if (!quiet)
-            simple_monster_message(this, " emerges from its shell.");
-        break;
-
     case ENCH_LIQUEFYING:
         invalidate_agrid();
 
         if (!quiet)
-            simple_monster_message(this, " is no longer liquefying the ground.");
+            simple_monster_message(*this, " is no longer liquefying the ground.");
         break;
 
     case ENCH_FLIGHT:
@@ -752,30 +749,24 @@ void monster::remove_enchantment_effect(const mon_enchant &me, bool quiet)
 
     case ENCH_DAZED:
         if (!quiet && alive())
-                simple_monster_message(this, " is no longer dazed.");
+                simple_monster_message(*this, " is no longer dazed.");
         break;
 
     case ENCH_INNER_FLAME:
         if (!quiet && alive())
-            simple_monster_message(this, "'s inner flame fades away.");
-        break;
-
-    case ENCH_ROLLING:
-        calc_speed();
-        if (!quiet && alive())
-            simple_monster_message(this, " stops rolling.");
+            simple_monster_message(*this, "'s inner flame fades away.");
         break;
 
     //The following should never happen, but just in case...
 
     case ENCH_MUTE:
         if (!quiet && alive())
-                simple_monster_message(this, " is no longer mute.");
+                simple_monster_message(*this, " is no longer mute.");
         break;
 
     case ENCH_BLIND:
         if (!quiet && alive())
-            simple_monster_message(this, " is no longer blind.");
+            simple_monster_message(*this, " is no longer blind.");
 
         // Reevaluate behaviour.
         behaviour_event(this, ME_EVAL);
@@ -783,7 +774,7 @@ void monster::remove_enchantment_effect(const mon_enchant &me, bool quiet)
 
     case ENCH_DUMB:
         if (!quiet && alive())
-            simple_monster_message(this, " is no longer stupefied.");
+            simple_monster_message(*this, " is no longer stupefied.");
 
         // Reevaluate behaviour.
         behaviour_event(this, ME_EVAL);
@@ -791,20 +782,23 @@ void monster::remove_enchantment_effect(const mon_enchant &me, bool quiet)
 
     case ENCH_MAD:
         if (!quiet && alive())
-            simple_monster_message(this, " is no longer mad.");
+            simple_monster_message(*this, " is no longer mad.");
 
         // Reevaluate behaviour.
         behaviour_event(this, ME_EVAL);
         break;
 
-    case ENCH_DEATHS_DOOR:
-        if (!quiet)
-            simple_monster_message(this, " is no longer invulnerable.");
-        break;
-
     case ENCH_REGENERATION:
         if (!quiet)
-            simple_monster_message(this, " is no longer regenerating.");
+            simple_monster_message(*this, " is no longer regenerating.");
+        break;
+
+    case ENCH_RAISED_MR:
+        if (!quiet)
+        {
+            simple_monster_message(*this, " is no longer resistant to hostile "
+                                          "enchantments.");
+        }
         break;
 
     case ENCH_WRETCHED:
@@ -813,7 +807,7 @@ void monster::remove_enchantment_effect(const mon_enchant &me, bool quiet)
             const string msg = " seems to return to " +
                                pronoun(PRONOUN_POSSESSIVE, true) +
                                " normal shape.";
-            simple_monster_message(this, msg.c_str());
+            simple_monster_message(*this, msg.c_str());
         }
         break;
 
@@ -832,18 +826,13 @@ void monster::remove_enchantment_effect(const mon_enchant &me, bool quiet)
 
     case ENCH_WEAK:
         if (!quiet)
-            simple_monster_message(this, " is no longer weakened.");
+            simple_monster_message(*this, " is no longer weakened.");
         break;
 
     case ENCH_AWAKEN_VINES:
         unawaken_vines(this, quiet);
         break;
 
-    case ENCH_CONTROL_WINDS:
-        if (!quiet && you.can_see(*this))
-            mprf("The winds cease moving at %s will.", name(DESC_ITS).c_str());
-        break;
-
     case ENCH_TOXIC_RADIANCE:
         if (!quiet && you.can_see(*this))
             mprf("%s toxic aura wanes.", name(DESC_ITS).c_str());
@@ -861,7 +850,7 @@ void monster::remove_enchantment_effect(const mon_enchant &me, bool quiet)
 
     case ENCH_FIRE_VULN:
         if (!quiet)
-            simple_monster_message(this, " is no longer more vulnerable to fire.");
+            simple_monster_message(*this, " is no longer more vulnerable to fire.");
         break;
 
     case ENCH_MERFOLK_AVATAR_SONG:
@@ -870,7 +859,7 @@ void monster::remove_enchantment_effect(const mon_enchant &me, bool quiet)
 
     case ENCH_POISON_VULN:
         if (!quiet)
-            simple_monster_message(this, " is no longer more vulnerable to poison.");
+            simple_monster_message(*this, " is no longer more vulnerable to poison.");
         break;
 
     case ENCH_ICEMAIL:
@@ -883,19 +872,19 @@ void monster::remove_enchantment_effect(const mon_enchant &me, bool quiet)
 
     case ENCH_AGILE:
         if (!quiet)
-            simple_monster_message(this, " is no longer unusually agile.");
+            simple_monster_message(*this, " is no longer unusually agile.");
         break;
 
     case ENCH_FROZEN:
         if (!quiet)
-            simple_monster_message(this, " is no longer encased in ice.");
+            simple_monster_message(*this, " is no longer encased in ice.");
         calc_speed();
         break;
 
     case ENCH_BLACK_MARK:
         if (!quiet)
         {
-            simple_monster_message(this, " is no longer absorbing vital"
+            simple_monster_message(*this, " is no longer absorbing vital"
                                          " energies.");
         }
         calc_speed();
@@ -903,47 +892,66 @@ void monster::remove_enchantment_effect(const mon_enchant &me, bool quiet)
 
     case ENCH_SAP_MAGIC:
         if (!quiet)
-            simple_monster_message(this, " is no longer being sapped.");
+            simple_monster_message(*this, " is no longer being sapped.");
         break;
 
     case ENCH_CORROSION:
         if (!quiet)
-           simple_monster_message(this, " is no longer covered in acid.");
+           simple_monster_message(*this, " is no longer covered in acid.");
         break;
 
     case ENCH_GOLD_LUST:
         if (!quiet)
-           simple_monster_message(this, " is no longer distracted by gold.");
+           simple_monster_message(*this, " is no longer distracted by gold.");
         break;
 
     case ENCH_DRAINED:
         if (!quiet)
-            simple_monster_message(this, " seems less drained.");
+            simple_monster_message(*this, " seems less drained.");
         break;
 
     case ENCH_REPEL_MISSILES:
         if (!quiet)
-            simple_monster_message(this, " is no longer repelling missiles.");
+            simple_monster_message(*this, " is no longer repelling missiles.");
         break;
 
     case ENCH_DEFLECT_MISSILES:
         if (!quiet)
-            simple_monster_message(this, " is no longer deflecting missiles.");
+            simple_monster_message(*this, " is no longer deflecting missiles.");
         break;
 
     case ENCH_RESISTANCE:
         if (!quiet)
-            simple_monster_message(this, " is no longer unusually resistant.");
+            simple_monster_message(*this, " is no longer unusually resistant.");
         break;
 
     case ENCH_BRILLIANCE_AURA:
         if (!quiet)
-            simple_monster_message(this, " is no longer giving off an aura.");
+            simple_monster_message(*this, " is no longer giving off an aura.");
         break;
 
     case ENCH_EMPOWERED_SPELLS:
         if (!quiet)
-            simple_monster_message(this, " seems less brilliant.");
+            simple_monster_message(*this, " seems less brilliant.");
+        break;
+
+    case ENCH_IDEALISED:
+        if (!quiet)
+            simple_monster_message(*this, " loses the glow of perfection.");
+        break;
+
+    case ENCH_BOUND_SOUL:
+        if (!quiet && you.can_see(*this))
+            mprf("%s soul is no longer bound.", name(DESC_ITS).c_str());
+        break;
+
+    case ENCH_INFESTATION:
+        if (!quiet)
+            simple_monster_message(*this, " is no longer infested.");
+        break;
+
+    case ENCH_STILL_WINDS:
+        end_still_winds();
         break;
 
     default:
@@ -1090,8 +1098,10 @@ static void _entangle_actor(actor* act)
     {
         you.duration[DUR_GRASPING_ROOTS] = 10;
         you.redraw_evasion = true;
-        if (you.duration[DUR_FLIGHT] ||  you.attribute[ATTR_PERM_FLIGHT])
+        if (you.duration[DUR_FLIGHT] || you.attribute[ATTR_PERM_FLIGHT])
         {
+            you.attribute[ATTR_LAST_FLIGHT_STATUS] =
+                you.attribute[ATTR_PERM_FLIGHT];
             you.duration[DUR_FLIGHT] = 0;
             you.attribute[ATTR_PERM_FLIGHT] = 0;
             land_player(true);
@@ -1119,11 +1129,8 @@ static bool _apply_grasping_roots(monster* mons)
     bool found_hostile = false;
     for (actor_near_iterator ai(mons, LOS_NO_TRANS); ai; ++ai)
     {
-        if (mons_aligned(mons, *ai) || ai->is_insubstantial()
-            || !ai->visible_to(mons))
-        {
+        if (mons_aligned(mons, *ai) || ai->is_insubstantial())
             continue;
-        }
 
         found_hostile = true;
 
@@ -1205,7 +1212,7 @@ static bool _merfolk_avatar_movement_effect(const monster* mons)
                 coord_def swapdest;
                 if (mon->wont_attack()
                     && !mon->is_stationary()
-                    && !mon->is_projectile()
+                    && !mons_is_projectile(*mon)
                     && !mon->cannot_act()
                     && !mon->asleep()
                     && swap_check(mon, swapdest, true))
@@ -1256,7 +1263,7 @@ static bool _merfolk_avatar_movement_effect(const monster* mons)
 static void _merfolk_avatar_song(monster* mons)
 {
     // First, attempt to pull the player, if mesmerised
-    if (you.beheld_by(mons) && coinflip())
+    if (you.beheld_by(*mons) && coinflip())
     {
         // Don't pull the player if they walked forward voluntarily this
         // turn (to avoid making you jump two spaces at once)
@@ -1273,9 +1280,9 @@ static void _merfolk_avatar_song(monster* mons)
     // Only call up drowned souls if we're largely alone; otherwise our
     // mesmerisation can support the present allies well enough.
     int ally_hd = 0;
-    for (monster_near_iterator mi(&you); mi; ++mi)
+    for (monster_near_iterator mi(you.pos()); mi; ++mi)
     {
-        if (*mi != mons && mons_aligned(mons, *mi) && mons_is_threatening(*mi)
+        if (*mi != mons && mons_aligned(mons, *mi) && mons_is_threatening(**mi)
             && mi->type != MONS_DROWNED_SOUL)
         {
             ally_hd += mi->get_experience_level();
@@ -1329,9 +1336,10 @@ static void _merfolk_avatar_song(monster* mons)
 
             for (int i = 0; i < num; ++i)
             {
-                monster* soul = create_monster(mgen_data(MONS_DROWNED_SOUL,
-                                 SAME_ATTITUDE(mons), mons, 1, SPELL_NO_SPELL,
-                                 deep_water[i], mons->foe, MG_FORCE_PLACE));
+                monster* soul = create_monster(
+                    mgen_data(MONS_DROWNED_SOUL, SAME_ATTITUDE(mons),
+                              deep_water[i], mons->foe, MG_FORCE_PLACE)
+                    .set_summoned(mons, 1, SPELL_NO_SPELL));
 
                 // Scale down drowned soul damage for low level merfolk avatars
                 if (soul)
@@ -1349,7 +1357,7 @@ void monster::apply_enchantment(const mon_enchant &me)
     case ENCH_INSANE:
         if (decay_enchantment(en))
         {
-            simple_monster_message(this, " is no longer in an insane frenzy.");
+            simple_monster_message(*this, " is no longer in an insane frenzy.");
             const int duration = random_range(70, 130);
             add_ench(mon_enchant(ENCH_FATIGUE, 0, 0, duration));
             add_ench(mon_enchant(ENCH_SLOW, 0, 0, duration));
@@ -1359,7 +1367,7 @@ void monster::apply_enchantment(const mon_enchant &me)
     case ENCH_BERSERK:
         if (decay_enchantment(en))
         {
-            simple_monster_message(this, " is no longer berserk.");
+            simple_monster_message(*this, " is no longer berserk.");
             const int duration = random_range(70, 130);
             add_ench(mon_enchant(ENCH_FATIGUE, 0, 0, duration));
             add_ench(mon_enchant(ENCH_SLOW, 0, 0, duration));
@@ -1369,22 +1377,11 @@ void monster::apply_enchantment(const mon_enchant &me)
     case ENCH_FATIGUE:
         if (decay_enchantment(en))
         {
-            simple_monster_message(this, " looks more energetic.");
+            simple_monster_message(*this, " looks more energetic.");
             del_ench(ENCH_SLOW, true);
         }
         break;
 
-    case ENCH_WITHDRAWN:
-        if (hit_points >= (max_hit_points - max_hit_points / 4)
-                && !one_chance_in(3))
-        {
-            del_ench(ENCH_WITHDRAWN);
-            break;
-        }
-
-        decay_enchantment(en);
-        break;
-
     case ENCH_SLOW:
     case ENCH_HASTE:
     case ENCH_SWIFT:
@@ -1403,7 +1400,7 @@ void monster::apply_enchantment(const mon_enchant &me)
     case ENCH_TIDE:
     case ENCH_REGENERATION:
     case ENCH_RAISED_MR:
-    case ENCH_MAGIC_ARMOUR:
+    case ENCH_IDEALISED:
     case ENCH_FEAR_INSPIRING:
     case ENCH_LIFE_TIMER:
     case ENCH_FLIGHT:
@@ -1434,12 +1431,16 @@ void monster::apply_enchantment(const mon_enchant &me)
     case ENCH_BRILLIANCE_AURA:
     case ENCH_EMPOWERED_SPELLS:
     case ENCH_ANTIMAGIC:
+    case ENCH_BOUND_SOUL:
+    case ENCH_INFESTATION:
+    case ENCH_BLACK_MARK:
+    case ENCH_STILL_WINDS:
         decay_enchantment(en);
         break;
 
     case ENCH_MIRROR_DAMAGE:
         if (decay_enchantment(en))
-            simple_monster_message(this, "'s dark mirror aura disappears.");
+            simple_monster_message(*this, "'s dark mirror aura disappears.");
         break;
 
     case ENCH_SILENCE:
@@ -1454,7 +1455,7 @@ void monster::apply_enchantment(const mon_enchant &me)
 
     case ENCH_AQUATIC_LAND:
         // Aquatic monsters lose hit points every turn they spend on dry land.
-        ASSERT(mons_habitat(this) == HT_WATER || mons_habitat(this) == HT_LAVA);
+        ASSERT(mons_habitat(*this) == HT_WATER || mons_habitat(*this) == HT_LAVA);
         if (monster_habitable_grid(this, grd(pos())))
         {
             del_ench(ENCH_AQUATIC_LAND);
@@ -1462,7 +1463,7 @@ void monster::apply_enchantment(const mon_enchant &me)
         }
 
         // Zombies don't take damage from flopping about on land.
-        if (mons_is_zombified(this))
+        if (mons_is_zombified(*this))
             break;
 
         hurt(me.agent(), 1 + random2(5), BEAM_NONE);
@@ -1535,15 +1536,16 @@ void monster::apply_enchantment(const mon_enchant &me)
 
         if (dam > 0)
         {
-            simple_monster_message(this, " burns!");
+            simple_monster_message(*this, " burns!");
             dprf("sticky flame damage: %d", dam);
 
-            if (type == MONS_SHEEP)
+            if (mons_genus(type) == MONS_SHEEP)
             {
                 for (adjacent_iterator ai(pos()); ai; ++ai)
                 {
                     monster *mon = monster_at(*ai);
-                    if (mon && mon->type == MONS_SHEEP
+                    if (mon
+                        && (mons_genus(mon->type) == MONS_SHEEP)
                         && !mon->has_ench(ENCH_STICKY_FLAME)
                         && coinflip())
                     {
@@ -1603,21 +1605,18 @@ void monster::apply_enchantment(const mon_enchant &me)
 
             for (fair_adjacent_iterator ai(pos()); ai; ++ai)
             {
-                if (mons_class_can_pass(MONS_GIANT_SPORE, grd(*ai))
+                if (mons_class_can_pass(MONS_BALLISTOMYCETE_SPORE, grd(*ai))
                     && !actor_at(*ai))
                 {
                     beh_type plant_attitude = SAME_ATTITUDE(this);
 
-                    if (monster *plant = create_monster(mgen_data(MONS_GIANT_SPORE,
+                    if (monster *plant = create_monster(mgen_data(MONS_BALLISTOMYCETE_SPORE,
                                                             plant_attitude,
-                                                            nullptr,
-                                                            0,
-                                                            0,
                                                             *ai,
                                                             MHITNOT,
                                                             MG_FORCE_PLACE)))
                     {
-                        if (mons_is_god_gift(this, GOD_FEDHAS))
+                        if (mons_is_god_gift(*this, GOD_FEDHAS))
                         {
                             plant->flags |= MF_NO_REWARD;
 
@@ -1625,7 +1624,7 @@ void monster::apply_enchantment(const mon_enchant &me)
                             {
                                 plant->flags |= MF_ATT_CHANGE_ATTEMPT;
 
-                                mons_make_god_gift(plant, GOD_FEDHAS);
+                                mons_make_god_gift(*plant, GOD_FEDHAS);
                             }
                         }
 
@@ -1633,7 +1632,7 @@ void monster::apply_enchantment(const mon_enchant &me)
                         plant->spore_cooldown = 20;
 
                         if (you.see_cell(*ai) && you.see_cell(pos()))
-                            mpr("A ballistomycete spawns a giant spore.");
+                            mpr("A ballistomycete spawns a ballistomycete spore.");
 
                         // Decrease the count and maybe become inactive
                         // again.
@@ -1725,7 +1724,7 @@ void monster::apply_enchantment(const mon_enchant &me)
             if (!silenced(you.pos()))
             {
                 if (you.can_see(*this))
-                    simple_monster_message(this, " suddenly becomes enraged!");
+                    simple_monster_message(*this, " suddenly becomes enraged!");
                 else
                     mpr("You hear a distant and violent thrashing sound.");
             }
@@ -1739,7 +1738,7 @@ void monster::apply_enchantment(const mon_enchant &me)
 
     case ENCH_SEVERED:
     {
-        simple_monster_message(this, " writhes!");
+        simple_monster_message(*this, " writhes!");
         coord_def base_position = props["base_position"].get_coord();
         maybe_bloodify_square(base_position);
         hurt(me.agent(), 20);
@@ -1776,9 +1775,6 @@ void monster::apply_enchantment(const mon_enchant &me)
             monster_teleport(this, true);
         break;
 
-    case ENCH_EAT_ITEMS:
-        break;
-
     case ENCH_AWAKEN_FOREST:
         forest_damage(this);
         decay_enchantment(en);
@@ -1803,7 +1799,7 @@ void monster::apply_enchantment(const mon_enchant &me)
         if (how_chaotic())
         {
             int dam = roll_dice(2, 4) - 1;
-            simple_monster_message(this, " is seared!");
+            simple_monster_message(*this, " is seared!");
             dprf("Zin's Corona damage: %d", dam);
             hurt(me.agent(), dam);
         }
@@ -1814,11 +1810,8 @@ void monster::apply_enchantment(const mon_enchant &me)
     case ENCH_WORD_OF_RECALL:
         // If we've gotten silenced or somehow incapacitated since we started,
         // cancel the recitation
-        if (silenced(pos()) || paralysed() || petrified()
-            || confused() || asleep() || has_ench(ENCH_FEAR)
-            || has_ench(ENCH_BREATH_WEAPON)
-            || has_ench(ENCH_WATER_HOLD) && !res_water_drowning()
-            || has_ench(ENCH_MUTE))
+        if (is_silenced() || cannot_act() || has_ench(ENCH_BREATH_WEAPON)
+            || confused() || asleep() || has_ench(ENCH_FEAR))
         {
             speed_increment += me.duration;
             del_ench(en, true, false);
@@ -1890,11 +1883,6 @@ void monster::apply_enchantment(const mon_enchant &me)
             del_ench(ENCH_HAUNTING);
         break;
 
-    case ENCH_CONTROL_WINDS:
-        apply_control_winds(this);
-        decay_enchantment(en);
-        break;
-
     case ENCH_TOXIC_RADIANCE:
         toxic_radiance_effect(this, 1);
         decay_enchantment(en);
@@ -1918,14 +1906,6 @@ void monster::apply_enchantment(const mon_enchant &me)
         }
         break;
 
-    case ENCH_DEATHS_DOOR:
-        if (decay_enchantment(en))
-        {
-            add_ench(mon_enchant(ENCH_FATIGUE, 0, 0,
-                                 (10 + random2(20)) * BASELINE_DELAY));
-        }
-        break;
-
     case ENCH_MERFOLK_AVATAR_SONG:
         // If we've gotten silenced or somehow incapacitated since we started,
         // cancel the song
@@ -1949,6 +1929,16 @@ void monster::apply_enchantment(const mon_enchant &me)
 
         break;
 
+    case ENCH_PAIN_BOND:
+        if (decay_enchantment(en))
+        {
+            const string msg = " is no longer sharing " +
+                               pronoun(PRONOUN_POSSESSIVE, true) +
+                               " pain.";
+            simple_monster_message(*this, msg.c_str());
+        }
+        break;
+
     default:
         break;
     }
@@ -2046,8 +2036,8 @@ void monster::apply_enchantments()
 static inline int _mod_speed(int val, int speed)
 {
     if (!speed)
-        speed = 10;
-    const int modded = val * 10 / speed;
+        speed = BASELINE_DELAY;
+    const int modded = val * BASELINE_DELAY / speed;
     return modded? modded : 1;
 }
 
@@ -2072,7 +2062,10 @@ static const char *enchant_names[] =
      "battle_frenzy", "temp_pacif",
 #endif
     "petrifying",
-    "petrified", "lowered_mr", "soul_ripe", "slowly_dying", "eat_items",
+    "petrified", "lowered_mr", "soul_ripe", "slowly_dying",
+#if TAG_MAJOR_VERSION == 34
+    "eat_items",
+#endif
     "aquatic_land", "spore_production",
 #if TAG_MAJOR_VERSION == 34
     "slouch",
@@ -2087,10 +2080,13 @@ static const char *enchant_names[] =
     "fading_away", "preparing_resurrect",
 #endif
     "regen",
-    "magic_res", "mirror_dam", "stoneskin", "fear inspiring", "temporarily pacified",
-    "withdrawn",
+    "magic_res", "mirror_dam",
 #if TAG_MAJOR_VERSION == 34
-    "attached",
+    "stoneskin",
+#endif
+    "fear inspiring", "temporarily pacified",
+#if TAG_MAJOR_VERSION == 34
+    "withdrawn", "attached",
 #endif
     "guardian_timer", "flight", "liquefying", "tornado", "fake_abjuration",
     "dazed", "mute", "blind", "dumb", "mad", "silver_corona", "recite timer",
@@ -2098,15 +2094,18 @@ static const char *enchant_names[] =
 #if TAG_MAJOR_VERSION == 34
     "roused",
 #endif
-    "breath timer", "deaths_door", "rolling",
+    "breath timer",
+#if TAG_MAJOR_VERSION == 34
+    "deaths_door", "rolling",
+#endif
     "ozocubus_armour", "wretched", "screamed", "rune_of_recall", "injury bond",
     "drowning", "flayed", "haunting",
 #if TAG_MAJOR_VERSION == 34
     "retching",
 #endif
-    "weak", "dimension_anchor", "awaken vines", "control_winds",
+    "weak", "dimension_anchor", "awaken vines",
 #if TAG_MAJOR_VERSION == 34
-    "wind_aided",
+    "control_winds", "wind_aided",
 #endif
     "summon_capped",
     "toxic_radiance", "grasping_roots_source", "grasping_roots",
@@ -2134,7 +2133,9 @@ static const char *enchant_names[] =
 #if TAG_MAJOR_VERSION == 34
     "chanting_fire_storm", "chanting_word_of_entropy",
 #endif
-    "aura_of_brilliance", "empowered_spells", "gozag_incite",
+    "aura_of_brilliance", "empowered_spells", "gozag_incite", "pain_bond",
+    "idealised", "bound_soul", "infestation",
+    "stilling the winds",
     "buggy",
 };
 
@@ -2205,7 +2206,8 @@ void mon_enchant::cap_degree()
 
     // Hard cap to simulate old enum behaviour, we should really throw this
     // out entirely.
-    const int max = (ench == ENCH_ABJ || ench == ENCH_FAKE_ABJURATION) ? 6 : 4;
+    const int max = (ench == ENCH_ABJ || ench == ENCH_FAKE_ABJURATION) ?
+            MAX_ENCH_DEGREE_ABJURATION : MAX_ENCH_DEGREE_DEFAULT;
     if (degree > max)
         degree = max;
 }
@@ -2265,10 +2267,6 @@ int mon_enchant::calc_duration(const monster* mons,
     // monster HD via modded_speed(). Use _mod_speed instead!
     switch (ench)
     {
-    case ENCH_WITHDRAWN:
-        cturn = 5000 / _mod_speed(25, mons->speed);
-        break;
-
     case ENCH_SWIFT:
         cturn = 1000 / _mod_speed(25, mons->speed);
         break;
@@ -2276,10 +2274,11 @@ int mon_enchant::calc_duration(const monster* mons,
     case ENCH_MIGHT:
     case ENCH_INVIS:
     case ENCH_FEAR_INSPIRING:
-    case ENCH_MAGIC_ARMOUR:
     case ENCH_AGILE:
     case ENCH_BLACK_MARK:
     case ENCH_RESISTANCE:
+    case ENCH_IDEALISED:
+    case ENCH_BOUND_SOUL:
         cturn = 1000 / _mod_speed(25, mons->speed);
         break;
     case ENCH_LIQUEFYING:
@@ -2287,8 +2286,8 @@ int mon_enchant::calc_duration(const monster* mons,
     case ENCH_REGENERATION:
     case ENCH_RAISED_MR:
     case ENCH_MIRROR_DAMAGE:
-    case ENCH_DEATHS_DOOR:
     case ENCH_SAP_MAGIC:
+    case ENCH_STILL_WINDS:
         cturn = 300 / _mod_speed(25, mons->speed);
         break;
     case ENCH_SLOW:
@@ -2336,7 +2335,7 @@ int mon_enchant::calc_duration(const monster* mons,
                   * speed_to_duration(mons->speed);
     case ENCH_SPORE_PRODUCTION:
         // This is used as a simple timer, when the enchantment runs out
-        // the monster will create a giant spore.
+        // the monster will create a ballistomycete spore.
         return random_range(475, 525) * 10;
 
     case ENCH_EXPLODING:
@@ -2387,9 +2386,6 @@ int mon_enchant::calc_duration(const monster* mons,
         return random_range(25, 35) * 10;
     case ENCH_BERSERK:
         return (16 + random2avg(13, 2)) * 10;
-    case ENCH_ROLLING:
-        cturn = 10000 / _mod_speed(25, mons->speed);
-        break;
     case ENCH_WRETCHED:
         cturn = (20 + roll_dice(3, 10)) * 10 / _mod_speed(10, mons->speed);
         break;
