@@ -84,6 +84,7 @@
 #include "mon-act.h"
 #include "mon-cast.h"
 #include "mon-death.h"
+#include "mon-place.h"
 #include "mon-util.h"
 #include "mutation.h"
 #include "options.h"
@@ -112,6 +113,7 @@
 #include "stash.h"
 #include "state.h"
 #include "status.h"
+#include "stepdown.h"
 #include "stringutil.h"
 #include "tags.h"
 #include "target.h"
@@ -291,7 +293,7 @@ static int _current_horror_level()
 
         if (mon == nullptr
             || mons_aligned(mon, &you)
-            || !mons_is_threatening(mon)
+            || !mons_is_threatening(*mon)
             || !you.can_see(*mon))
         {
             continue;
@@ -299,7 +301,7 @@ static int _current_horror_level()
 
         ASSERT(mon);
 
-        const mon_threat_level_type threat_level = mons_threat_level(mon);
+        const mon_threat_level_type threat_level = mons_threat_level(*mon);
         if (threat_level == MTHRT_NASTY)
             horror_level += 3;
         else if (threat_level == MTHRT_TOUGH)
@@ -377,6 +379,74 @@ static void _update_cowardice()
         mpr("You feel a twist of horror at the sight of this foe.");
 }
 
+// Uskawyaw piety decays incredibly fast, but only to a baseline level of *.
+// Using Uskayaw abilities can still take you under *.
+static void _handle_uskayaw_piety(int time_taken)
+{
+    if (you.props[USKAYAW_NUM_MONSTERS_HURT].get_int() > 0)
+    {
+        int num_hurt = you.props[USKAYAW_NUM_MONSTERS_HURT];
+        int hurt_val = you.props[USKAYAW_MONSTER_HURT_VALUE];
+        int piety_gain = max(num_hurt, stepdown_value(hurt_val, 5, 10, 20, 40));
+
+        gain_piety(piety_gain);
+        you.props[USKAYAW_AUT_SINCE_PIETY_GAIN] = 0;
+    }
+    else if (you.piety > piety_breakpoint(0))
+    {
+        // If we didn't do a dance action and we can lose piety, we're going
+        // to lose piety proportional to the time since the last time we took
+        // a dance action and hurt a monster.
+        int time_since_gain = you.props[USKAYAW_AUT_SINCE_PIETY_GAIN].get_int();
+        time_since_gain += time_taken;
+
+        // Only start losing piety if it's been a few turns since we gained
+        // piety, in order to give more tolerance for missing in combat.
+        if (time_since_gain > 30)
+        {
+            int piety_lost = min(you.piety - piety_breakpoint(0),
+                    div_rand_round(time_since_gain, 10));
+
+            if (piety_lost > 0)
+                lose_piety(piety_lost);
+
+        }
+        you.props[USKAYAW_AUT_SINCE_PIETY_GAIN] = time_since_gain;
+    }
+
+    // Re-initialize Uskayaw piety variables
+    you.props[USKAYAW_NUM_MONSTERS_HURT] = 0;
+    you.props[USKAYAW_MONSTER_HURT_VALUE] = 0;
+}
+
+static void _handle_uskayaw_time(int time_taken)
+{
+    _handle_uskayaw_piety(time_taken);
+
+    int audience_timer = you.props[USKAYAW_AUDIENCE_TIMER].get_int();
+    int bond_timer = you.props[USKAYAW_BOND_TIMER].get_int();
+
+    // For the timered abilities, if we set the timer to -1, that means we
+    // need to trigger the abilities this turn. Otherwise we'll decrement the
+    // timer down to a minimum of 0, at which point it becomes eligible to
+    // trigger again.
+    if (audience_timer == -1 || (you.piety >= piety_breakpoint(2)
+            && x_chance_in_y(time_taken, time_taken * 10 + audience_timer)))
+    {
+        uskayaw_prepares_audience();
+    }
+    else
+        you.props[USKAYAW_AUDIENCE_TIMER] = max(0, audience_timer - time_taken);
+
+    if (bond_timer == -1 || (you.piety >= piety_breakpoint(3)
+            && x_chance_in_y(time_taken, time_taken * 10 + bond_timer)))
+    {
+        uskayaw_bonds_audience();
+    }
+    else
+        you.props[USKAYAW_BOND_TIMER] =  max(0, bond_timer - time_taken);
+}
+
 /**
  * Player reactions after monster and cloud activities in the turn are finished.
  */
@@ -394,18 +464,14 @@ void player_reacts_to_monsters()
     if (have_passive(passive_t::detect_items) || you.mutation[MUT_JELLY_GROWTH])
         detect_items(-1);
 
-    if (you.duration[DUR_TELEPATHY])
-    {
-        detect_creatures(1 + you.duration[DUR_TELEPATHY] /
-                         (2 * BASELINE_DELAY), true);
-    }
-
     _decrement_paralysis(you.time_taken);
     _decrement_petrification(you.time_taken);
     if (_decrement_a_duration(DUR_SLEEP, you.time_taken))
-        you.awake();
+        you.awaken();
     _maybe_melt_armour();
     _update_cowardice();
+    if (you_worship(GOD_USKAYAW))
+        _handle_uskayaw_time(you.time_taken);
 }
 
 static bool _check_recite()
@@ -437,7 +503,7 @@ static void _handle_recitation(int step)
     // Recite trains more than once per use, because it has a
     // long timer in between uses and actually takes up multiple
     // turns.
-    practise(EX_USED_ABIL, ABIL_ZIN_RECITE);
+    practise_using_ability(ABIL_ZIN_RECITE);
 
     noisy(you.shout_volume(), you.pos());
 
@@ -459,6 +525,22 @@ static void _handle_recitation(int step)
     }
 }
 
+/**
+ * Try to respawn the player's ancestor, if possible.
+ */
+static void _try_to_respawn_ancestor()
+{
+     monster *ancestor = create_monster(hepliaklqana_ancestor_gen_data());
+     if (!ancestor)
+         return;
+
+    mprf("%s emerges from the mists of memory!",
+         ancestor->name(DESC_YOUR).c_str());
+    add_companion(ancestor);
+    check_place_cloud(CLOUD_MIST, ancestor->pos(), random_range(1,2),
+                      ancestor); // ;)
+}
+
 
 /**
  * Take a 'simple' duration, decrement it, and print messages as appropriate
@@ -556,6 +638,13 @@ static void _decrement_durations()
         || you.duration[DUR_TRANSFORMATION] <= 5 * BASELINE_DELAY
         || you.transform_uncancellable)
     {
+        if (form_can_fly()
+            || form_likes_water() && feat_is_water(grd(you.pos())))
+        {
+            // Disable emergency flight if it was active
+            you.props.erase(EMERGENCY_FLIGHT_KEY);
+        }
+
         if (_decrement_a_duration(DUR_TRANSFORMATION, delay, nullptr, random2(3),
                                   "Your transformation is almost over."))
         {
@@ -584,30 +673,16 @@ static void _decrement_durations()
         }
     }
 
-    // Handle Powered By Death strength and duration
+    // Decrement Powered By Death strength
     int pbd_str = you.props[POWERED_BY_DEATH_KEY].get_int();
-    if (pbd_str > 1)
+    if (pbd_str > 0 && _decrement_a_duration(DUR_POWERED_BY_DEATH, delay))
     {
-        // Roll to decrement (on average) 1 per-10 aut.
-        const int decrement_rolls = div_rand_round(delay, 10);
-        const int dec = binomial(decrement_rolls, 1, 4);
-
-        // We don't want to accidentally terminate the effect after slow actions
-        pbd_str = max(pbd_str - dec, 1);
-        you.props[POWERED_BY_DEATH_KEY] = pbd_str;
-        if (dec > 0)
-            dprf("Decrementing Powered by Death strength to %d", pbd_str);
-    }
-    if (_decrement_a_duration(DUR_POWERED_BY_DEATH, delay))
-    {
-        if (pbd_str > 0)
-        {
-            mprf(MSGCH_DURATION, "You feel less regenerative.");
-            you.props[POWERED_BY_DEATH_KEY] = 0;
-        }
+        you.props[POWERED_BY_DEATH_KEY] = pbd_str - 1;
+        reset_powered_by_death_duration();
     }
 
     dec_ambrosia_player(delay);
+    dec_channel_player(delay);
     dec_slow_player(delay);
     dec_exhaust_player(delay);
     dec_haste_player(delay);
@@ -626,13 +701,6 @@ static void _decrement_durations()
         }
     }
 
-
-    if (you.duration[DUR_BERSERK]
-        && you.hunger + 100 <= HUNGER_STARVING + BERSERK_NUTRITION)
-    {
-        you.duration[DUR_BERSERK] = 1; // end
-    }
-
     // Leak piety from the piety pool into actual piety.
     // Note that changes of religious status without corresponding actions
     // (killing monsters, offering items, ...) might be confusing for characters
@@ -675,12 +743,18 @@ static void _decrement_durations()
             {
                 land_player();
             }
+            else
+            {
+                // Disable emergency flight if it was active
+                you.props.erase(EMERGENCY_FLIGHT_KEY);
+            }
         }
         else if ((you.duration[DUR_FLIGHT] -= delay) <= 0)
         {
             // Just time out potions/spells/miscasts.
             you.attribute[ATTR_FLIGHT_UNCANCELLABLE] = 0;
             you.duration[DUR_FLIGHT] = 0;
+            you.props.erase(EMERGENCY_FLIGHT_KEY);
         }
     }
 
@@ -689,8 +763,12 @@ static void _decrement_durations()
         set_hp(allowed_deaths_door_hp());
         you.redraw_hit_points = true;
     }
-    // XXX: this should probably be changed to be by aut rather than turns vvv
-    _decrement_a_duration(DUR_COLOUR_SMOKE_TRAIL, 1);
+
+    if (_decrement_a_duration(DUR_CLOUD_TRAIL, delay,
+            "Your trail of clouds dissipates."))
+    {
+        you.props.erase(XOM_CLOUD_TRAIL_TYPE_KEY);
+    }
 
     if (you.duration[DUR_DARKNESS] && you.haloed())
     {
@@ -757,11 +835,24 @@ static void _decrement_durations()
         doom_howl(min(delay, you.duration[DUR_DOOM_HOWL]));
 
     dec_elixir_player(delay);
-    you.maybe_degrade_bone_armour(1, delay);
+    you.maybe_degrade_bone_armour(delay);
 
     if (!env.sunlight.empty())
         process_sunlights();
 
+    if (!you.duration[DUR_ANCESTOR_DELAY]
+        && in_good_standing(GOD_HEPLIAKLQANA)
+        && hepliaklqana_ancestor() == MID_NOBODY)
+    {
+        _try_to_respawn_ancestor();
+    }
+
+    const bool sanguine_armour_is_valid = sanguine_armour_valid();
+    if (sanguine_armour_is_valid)
+        activate_sanguine_armour();
+    else if (!sanguine_armour_is_valid && you.duration[DUR_SANGUINE_ARMOUR])
+        you.duration[DUR_SANGUINE_ARMOUR] = 1; // expire
+
     // these should be after decr_ambrosia, transforms, liquefying, etc.
     for (int i = 0; i < NUM_DURATIONS; ++i)
         if (duration_decrements_normally((duration_type) i))
@@ -815,6 +906,23 @@ static void _rot_ghoul_players()
     }
 }
 
+static void _handle_emergency_flight()
+{
+    ASSERT(you.props[EMERGENCY_FLIGHT_KEY].get_bool());
+
+    if (!is_feat_dangerous(orig_terrain(you.pos()), true, false))
+    {
+        mpr("You float gracefully downwards.");
+        land_player();
+        you.props.erase(EMERGENCY_FLIGHT_KEY);
+    }
+    else
+    {
+        const int drain = div_rand_round(15 * you.time_taken, BASELINE_DELAY);
+        drain_player(drain, true, true);
+    }
+}
+
 // cjo: Handles player hp and mp regeneration. If the counter
 // you.hit_points_regeneration is over 100, a loop restores 1 hp and decreases
 // the counter by 100 (so you can regen more than 1 hp per turn). If the counter
@@ -861,7 +969,7 @@ static void _regenerate_hp_and_mp(int delay)
         if (player_mutation_level(MUT_MANA_REGENERATION))
             mp_regen_countup *= 2;
         if (you.wearing(EQ_AMULET, AMU_MANA_REGENERATION))
-            mp_regen_countup += 20;
+            mp_regen_countup += div_rand_round(15 * delay, BASELINE_DELAY);
 
         you.magic_points_regeneration += mp_regen_countup;
     }
@@ -882,7 +990,7 @@ void player_reacts()
     search_around();
 
     //XXX: does this _need_ to be calculated up here?
-    const int stealth = check_stealth();
+    const int stealth = player_stealth();
 
 #ifdef DEBUG_STEALTH
     // Too annoying for regular diagnostics.
@@ -916,7 +1024,7 @@ void player_reacts()
     if (you.duration[DUR_SONG_OF_SLAYING])
         noisy(spell_effect_noise(SPELL_SONG_OF_SLAYING), you.pos());
 
-    if (one_chance_in(10))
+    if (x_chance_in_y(you.time_taken, 10 * BASELINE_DELAY))
     {
         const int teleportitis_level = player_teleport();
         // this is instantaneous
@@ -946,7 +1054,7 @@ void player_reacts()
 
     // Icy shield and armour melt over lava.
     if (grd(you.pos()) == DNGN_LAVA)
-        maybe_melt_player_enchantments(BEAM_FIRE, 10);
+        maybe_melt_player_enchantments(BEAM_FIRE, you.time_taken);
 
     // Handle starvation before subtracting hunger for this turn (including
     // hunger from the berserk duration) and before monsters react, so you
@@ -993,6 +1101,9 @@ void player_reacts()
         xom_tick();
     else if (you_worship(GOD_QAZLAL))
         qazlal_storm_clouds();
+
+    if (you.props[EMERGENCY_FLIGHT_KEY].get_bool())
+        _handle_emergency_flight();
 }
 
 void extract_manticore_spikes(const char* endmsg)
