@@ -29,12 +29,14 @@ ranged_attack::ranged_attack(actor *attk, actor *defn, item_def *proj,
                              bool tele, actor *blame)
     : ::attack(attk, defn, blame), range_used(0), reflected(false),
       projectile(proj), teleport(tele), orig_to_hit(0),
-      should_alert_defender(true)
+      should_alert_defender(true), launch_type(LRET_BUGGY)
 {
     init_attack(SK_THROWING, 0);
     kill_type = KILLED_BY_BEAM;
 
     string proj_name = projectile->name(DESC_PLAIN);
+    // init launch type early, so we can use it later in the constructor
+    launch_type = is_launched(attacker, weapon, *projectile);
 
     // [dshaligram] When changing bolt names here, you must edit
     // hiscores.cc (scorefile_entry::terse_missile_cause()) to match.
@@ -43,7 +45,7 @@ ranged_attack::ranged_attack(actor *attk, actor *defn, item_def *proj,
         kill_type = KILLED_BY_SELF_AIMED;
         aux_source = proj_name;
     }
-    else if (is_launched(attacker, weapon, *projectile) == LRET_LAUNCHED)
+    else if (launch_type == LRET_LAUNCHED)
     {
         aux_source = make_stringf("Shot with a%s %s by %s",
                  (is_vowel(proj_name[0]) ? "n" : ""), proj_name.c_str(),
@@ -65,6 +67,10 @@ ranged_attack::ranged_attack(actor *attk, actor *defn, item_def *proj,
 int ranged_attack::calc_to_hit(bool random)
 {
     orig_to_hit = attack::calc_to_hit(random);
+
+    if (orig_to_hit == AUTOMATIC_HIT)
+        return AUTOMATIC_HIT;
+
     if (teleport)
     {
         orig_to_hit +=
@@ -144,7 +150,7 @@ bool ranged_attack::attack()
     if (attacker->is_player() && defender->is_monster()
         && !shield_blocked && ev_margin >= 0)
     {
-        print_wounds(defender->as_monster());
+        print_wounds(*defender->as_monster());
     }
 
     handle_phase_end();
@@ -298,8 +304,7 @@ bool ranged_attack::handle_phase_hit()
         }
     }
 
-    if (using_weapon()
-        || is_launched(attacker, weapon, *projectile) == LRET_THROWN)
+    if (using_weapon() || launch_type == LRET_THROWN)
     {
         if (using_weapon()
             && apply_damage_brand(projectile->name(DESC_THE).c_str()))
@@ -323,13 +328,14 @@ bool ranged_attack::handle_phase_hit()
 
 bool ranged_attack::using_weapon()
 {
-    return weapon
-           && is_launched(attacker, weapon, *projectile) == LRET_LAUNCHED;
+    return weapon && (launch_type == LRET_LAUNCHED
+                     || launch_type == LRET_BUGGY // not initialized
+                         && is_launched(attacker, weapon, *projectile));
 }
 
 int ranged_attack::weapon_damage()
 {
-    if (is_launched(attacker, weapon, *projectile) == LRET_FUMBLED)
+    if (launch_type == LRET_FUMBLED)
         return 0;
 
     int dam = property(*projectile, PWPN_DAMAGE);
@@ -355,7 +361,7 @@ int ranged_attack::weapon_damage()
 int ranged_attack::calc_base_unarmed_damage()
 {
     // No damage bonus for throwing non-throwing weapons.
-    if (is_launched(attacker, weapon, *projectile) == LRET_FUMBLED)
+    if (launch_type == LRET_FUMBLED)
         return 0;
 
     int damage = you.skill_rdiv(wpn_skill);
@@ -443,8 +449,7 @@ special_missile_type ranged_attack::random_chaos_missile_brand()
                      5, SPMSL_FRENZY,
                      2, SPMSL_CURARE,
                      2, SPMSL_CONFUSION,
-                     2, SPMSL_DISPERSAL,
-                     0));
+                     2, SPMSL_DISPERSAL));
 
         if (one_chance_in(3))
             break;
@@ -530,7 +535,7 @@ bool ranged_attack::blowgun_check(special_missile_type type)
         {
             if (defender->is_monster())
             {
-                simple_monster_message(defender->as_monster(),
+                simple_monster_message(*defender->as_monster(),
                                        " is unaffected.");
             }
             else
@@ -573,7 +578,7 @@ bool ranged_attack::blowgun_check(special_missile_type type)
         if (needs_message)
         {
             if (defender->is_monster())
-                simple_monster_message(defender->as_monster(), " resists.");
+                simple_monster_message(*defender->as_monster(), " resists.");
             else
                 canned_msg(MSG_YOU_RESIST);
         }
@@ -638,11 +643,6 @@ bool ranged_attack::apply_missile_brand()
     default:
         break;
     case SPMSL_FLAME:
-        if (using_weapon()
-            && get_weapon_brand(*weapon) == SPWPN_FREEZING)
-        {
-            break;
-        }
         calc_elemental_brand_damage(BEAM_FIRE,
                                     defender->is_icy() ? "melt" : "burn",
                                     projectile->name(DESC_THE).c_str());
@@ -654,19 +654,14 @@ bool ranged_attack::apply_missile_brand()
         attacker->god_conduct(DID_FIRE, 1);
         break;
     case SPMSL_FROST:
-        if (using_weapon()
-            && get_weapon_brand(*weapon) == SPWPN_FLAMING)
-        {
-            break;
-        }
         calc_elemental_brand_damage(BEAM_COLD, "freeze",
                                     projectile->name(DESC_THE).c_str());
         defender->expose_to_element(BEAM_COLD, 2);
         break;
     case SPMSL_POISONED:
         if (projectile->is_type(OBJ_MISSILES, MI_NEEDLE)
-            && using_weapon()
-            && damage_done > 0
+                && using_weapon()
+                && damage_done > 0
             || !one_chance_in(4))
         {
             int old_poison;
