@@ -71,12 +71,6 @@
 //
 spret_type cast_delayed_fireball(bool fail)
 {
-    if (you.attribute[ATTR_DELAYED_FIREBALL])
-    {
-        mpr("You are already charged.");
-        return SPRET_ABORT;
-    }
-
     fail_check();
     // Okay, this message is weak but functional. - bwr
     mpr("You feel magically charged.");
@@ -122,7 +116,6 @@ spret_type cast_fire_storm(int pow, bolt &beam, bool fail)
         return SPRET_ABORT;
     }
 
-    fail_check();
     setup_fire_storm(&you, pow, beam);
 
     bolt tempbeam = beam;
@@ -133,6 +126,8 @@ spret_type cast_fire_storm(int pow, bolt &beam, bool fail)
     if (tempbeam.beam_cancelled)
         return SPRET_ABORT;
 
+    fail_check();
+
     beam.refine_for_explosion();
     beam.explode(false);
 
@@ -393,7 +388,7 @@ static bool _refrigerateable_hitfunc(const actor *act)
     return _refrigerateable(&you, act);
 }
 
-static void _pre_refrigerate(actor* agent, bool player,
+static void _pre_refrigerate(const actor* agent, bool player,
                              vector<monster *> affected_monsters)
 {
     if (!affected_monsters.empty())
@@ -432,8 +427,8 @@ static const dice_def _refrigerate_damage(int pow)
     return dice_def(3, 5 + pow / 10);
 }
 
-static int _refrigerate_player(actor* agent, int pow, int avg,
-                               bool actual, bool added_effects)
+static int _refrigerate_player(const actor* agent, int pow, int avg,
+                               bool actual)
 {
     const dice_def dam_dice = _refrigerate_damage(pow);
 
@@ -447,7 +442,7 @@ static int _refrigerate_player(actor* agent, int pow, int avg,
             ouch(hurted, KILLED_BY_BEAM, agent->mid,
                  "by Ozocubu's Refrigeration", true,
                  agent->as_monster()->name(DESC_A).c_str());
-            you.expose_to_element(BEAM_COLD, 5, added_effects);
+            you.expose_to_element(BEAM_COLD, 5);
 
             // Note: this used to be 12!... and it was also applied even if
             // the player didn't take damage from the cold, so we're being
@@ -456,7 +451,7 @@ static int _refrigerate_player(actor* agent, int pow, int avg,
         else
         {
             ouch(hurted, KILLED_BY_FREEZING);
-            you.expose_to_element(BEAM_COLD, 5, added_effects);
+            you.expose_to_element(BEAM_COLD, 5);
             you.increase_duration(DUR_NO_POTIONS, 7 + random2(9), 15);
         }
     }
@@ -464,8 +459,8 @@ static int _refrigerate_player(actor* agent, int pow, int avg,
     return hurted;
 }
 
-static int _refrigerate_monster(actor* agent, monster* target, int pow, int avg,
-                                bool actual, bool added_effects)
+static int _refrigerate_monster(const actor* agent, monster* target, int pow,
+                                int avg, bool actual)
 {
     const dice_def dam_dice = _refrigerate_damage(pow);
 
@@ -491,7 +486,7 @@ static int _refrigerate_monster(actor* agent, monster* target, int pow, int avg,
         }
 
         if (target->alive() && you.can_see(*target))
-            print_wounds(target);
+            print_wounds(*target);
 
         if (agent && agent->is_player()
             && (is_sanctuary(you.pos()) || is_sanctuary(target->pos())))
@@ -509,7 +504,7 @@ static int _refrigerate_monster(actor* agent, monster* target, int pow, int avg,
 
 static bool _drain_lifeable(const actor* agent, const actor* act)
 {
-    if (act->res_negative_energy())
+    if (act->res_negative_energy() >= 3)
         return false;
 
     if (!agent)
@@ -528,90 +523,75 @@ static bool _drain_lifeable_hitfunc(const actor* act)
     return _drain_lifeable(&you, act);
 }
 
-static int _drain_player(actor* agent, int pow, int avg,
-                         bool actual, bool added_effects)
+static int _drain_player(const actor* agent, int pow, int avg, bool actual)
 {
+    const int hurted = resist_adjust_damage(&you, BEAM_NEG, avg);
     if (actual)
     {
-        monster* mons = agent ? agent->as_monster() : 0;
-        ouch(avg, KILLED_BY_BEAM, mons ? mons->mid : MID_NOBODY,
+        const monster* mons = agent ? agent->as_monster() : 0;
+        ouch(hurted, KILLED_BY_BEAM, mons ? mons->mid : MID_NOBODY,
              "by drain life");
     }
 
-    return avg;
+    return hurted;
 }
 
-static int _drain_monster(actor* agent, monster* target, int pow, int avg,
-                          bool actual, bool added_effects)
+static int _drain_monster(const actor* agent, monster* target, int pow,
+                          int avg, bool actual)
 {
     ASSERT(target); // XXX: change to monster &target
+    int hurted = resist_adjust_damage(target, BEAM_NEG, avg);
     if (actual)
     {
-        if (agent && agent->is_player())
+        if (hurted)
         {
-            mprf("You draw life from %s.",
-                 target->name(DESC_THE).c_str());
+            if (agent && agent->is_player())
+            {
+                mprf("You draw life from %s.",
+                     target->name(DESC_THE).c_str());
+            }
+            target->hurt(agent, hurted);
         }
 
-        behaviour_event(target, ME_ANNOY, agent,
-                        agent ? agent->pos() : coord_def(0, 0));
-
-        target->hurt(agent, avg);
+        if (target->alive())
+        {
+            behaviour_event(target, ME_ANNOY, agent,
+                            agent ? agent->pos() : coord_def(0, 0));
+        }
 
         if (target->alive() && you.can_see(*target))
-            print_wounds(target);
+            print_wounds(*target);
     }
 
     if (!target->is_summoned())
-        return avg;
+        return hurted;
 
     return 0;
 }
 
-static void _post_drain_life(actor* agent, bool player,
-                             vector<monster *> affected_monsters,
-                             int pow, int total_damage)
+static spret_type _cast_los_attack_spell(spell_type spell, int pow, const
+                                         actor* agent, bool actual, bool fail,
+                                         int* damage_done)
 {
-    total_damage /= 2;
-
-    total_damage = min(pow * 2, total_damage);
-
-    if (total_damage && agent)
-    {
-        if (agent->is_player())
-        {
-            mpr("You feel life flooding into your body.");
-            inc_hp(total_damage);
-        }
-        else
-        {
-            monster* mons = agent->as_monster();
-            ASSERT(mons);
-            if (mons->heal(total_damage))
-                simple_monster_message(mons, " is healed.");
-        }
-    }
-}
-
-spret_type cast_los_attack_spell(spell_type spell, int pow, actor* agent,
-                                 bool actual, bool added_effects, bool fail,
-                                 bool allow_cancel)
-{
-    monster* mons = agent ? agent->as_monster() : nullptr;
+    const monster* mons = agent ? agent->as_monster() : nullptr;
 
     colour_t flash_colour = BLACK;
     const char *player_msg = nullptr, *global_msg = nullptr,
                *mons_vis_msg = nullptr, *mons_invis_msg = nullptr;
     bool (*vulnerable)(const actor *, const actor *) = nullptr;
     bool (*vul_hitfunc)(const actor *) = nullptr;
-    int (*damage_player)(actor *, int, int, bool, bool) = nullptr;
-    int (*damage_monster)(actor *, monster *, int, int, bool, bool) = nullptr;
-    void (*pre_hook)(actor*, bool, vector<monster *>) = nullptr;
-    void (*post_hook)(actor*, bool, vector<monster *>, int, int) = nullptr;
+    int (*damage_player)(const actor *, int, int, bool) = nullptr;
+    int (*damage_monster)(const actor *, monster *, int, int, bool)
+        = nullptr;
+    void (*pre_hook)(const actor*, bool, vector<monster *>) = nullptr;
+    int fake_damage = -1;
+    if (!damage_done)
+        damage_done = &fake_damage;
 
     int hurted = 0;
     int this_damage = 0;
     int total_damage = 0;
+    *damage_done = total_damage;
 
     bolt beam;
     beam.source_id = agent ? agent->mid : MID_NOBODY;
@@ -645,7 +625,6 @@ spret_type cast_los_attack_spell(spell_type spell, int pow, actor* agent,
             vul_hitfunc = &_drain_lifeable_hitfunc;
             damage_player = &_drain_player;
             damage_monster = &_drain_monster;
-            post_hook = &_post_drain_life;
             hurted = 3 + random2(7) + random2(pow);
             break;
 
@@ -657,7 +636,7 @@ spret_type cast_los_attack_spell(spell_type spell, int pow, actor* agent,
         ASSERT(actual);
         targetter_los hitfunc(&you, LOS_NO_TRANS);
         {
-            if (allow_cancel && stop_attack_prompt(hitfunc, "harm", vul_hitfunc))
+            if (stop_attack_prompt(hitfunc, "harm", vul_hitfunc))
                 return SPRET_ABORT;
         }
         fail_check();
@@ -673,7 +652,7 @@ spret_type cast_los_attack_spell(spell_type spell, int pow, actor* agent,
         if (!agent)
             mpr(global_msg);
         else if (you.can_see(*agent))
-            simple_monster_message(mons, mons_vis_msg);
+            simple_monster_message(*mons, mons_vis_msg);
         else if (you.see_cell(agent->pos()))
             mpr(mons_invis_msg);
 
@@ -699,8 +678,7 @@ spret_type cast_los_attack_spell(spell_type spell, int pow, actor* agent,
     // order from the original behaviour in the case of refrigerate.
     if (affects_you)
     {
-        total_damage = (*damage_player)(agent, pow, hurted, actual,
-                                        added_effects);
+        total_damage = (*damage_player)(agent, pow, hurted, actual);
         if (!actual && mons)
         {
             if (mons->wont_attack())
@@ -724,12 +702,11 @@ spret_type cast_los_attack_spell(spell_type spell, int pow, actor* agent,
     for (auto m : affected_monsters)
     {
         // Watch out for invalidation. Example: Ozocubu's refrigeration on
-        // a bunch of giant spores that blow each other up.
+        // a bunch of ballistomycete spores that blow each other up.
         if (!m->alive())
             continue;
 
-        this_damage = (*damage_monster)(agent, m, pow, hurted, actual,
-                                        added_effects);
+        this_damage = (*damage_monster)(agent, m, pow, hurted, actual);
         total_damage += this_damage;
 
         if (!actual && mons)
@@ -747,20 +724,23 @@ spret_type cast_los_attack_spell(spell_type spell, int pow, actor* agent,
         }
     }
 
+    *damage_done = total_damage;
     if (actual)
-    {
-        if (post_hook)
-        {
-            (*post_hook)(agent, affects_you, affected_monsters, pow,
-                         total_damage);
-        }
-
         return SPRET_SUCCESS;
-    }
-
     return mons_should_fire(beam) ? SPRET_SUCCESS : SPRET_ABORT;
 }
 
+spret_type trace_los_attack_spell(spell_type spell, int pow, const actor* agent)
+{
+    return _cast_los_attack_spell(spell, pow, agent, false, false, nullptr);
+}
+
+spret_type fire_los_attack_spell(spell_type spell, int pow, const actor* agent,
+                                 bool fail, int* damage_done)
+{
+    return _cast_los_attack_spell(spell, pow, agent, true, fail, damage_done);
+}
+
 // Screaming Sword
 void sonic_damage(bool scream)
 {
@@ -867,18 +847,14 @@ spret_type vampiric_drain(int pow, monster* mons, bool fail)
         return SPRET_SUCCESS;
     }
 
-    if (mons->res_negative_energy())
-    {
-        canned_msg(MSG_NOTHING_HAPPENS);
-        return SPRET_SUCCESS;
-    }
-
     // The practical maximum of this is about 25 (pow @ 100). - bwr
     int hp_gain = 3 + random2avg(9, 2) + random2(pow) / 7;
 
     hp_gain = min(mons->hit_points, hp_gain);
     hp_gain = min(you.hp_max - you.hp, hp_gain);
 
+    hp_gain = resist_adjust_damage(mons, BEAM_NEG, hp_gain);
+
     if (!hp_gain)
     {
         canned_msg(MSG_NOTHING_HAPPENS);
@@ -890,7 +866,7 @@ spret_type vampiric_drain(int pow, monster* mons, bool fail)
     mons->hurt(&you, hp_gain);
 
     if (mons->alive())
-        print_wounds(mons);
+        print_wounds(*mons);
 
     hp_gain = div_rand_round(hp_gain, 2);
 
@@ -951,7 +927,7 @@ spret_type cast_freeze(int pow, monster* mons, bool fail)
     if (mons->alive())
     {
         mons->expose_to_element(BEAM_COLD, orig_hurted);
-        print_wounds(mons);
+        print_wounds(*mons);
     }
 
     return SPRET_SUCCESS;
@@ -1021,7 +997,7 @@ spret_type cast_airstrike(int pow, const dist &beam, bool fail)
 
     mons->hurt(&you, hurted);
     if (mons->alive())
-        print_wounds(mons);
+        print_wounds(*mons);
 
     return SPRET_SUCCESS;
 }
@@ -1037,7 +1013,7 @@ static bool _player_hurt_monster(monster& m, int damage,
 
         if (m.alive())
         {
-            print_wounds(&m);
+            print_wounds(m);
             behaviour_event(&m, ME_WHACK, &you);
         }
         else
@@ -1068,7 +1044,9 @@ static int _shatter_mon_dice(const monster *mon)
     case MONS_GARGOYLE:
     case MONS_IRON_ELEMENTAL:
     case MONS_IRON_GOLEM:
+    case MONS_PEACEKEEPER:
     case MONS_WAR_GARGOYLE:
+    case MONS_SALTLING:
     case MONS_CRYSTAL_GUARDIAN:
     case MONS_OBSIDIAN_STATUE:
     case MONS_ORANGE_STATUE:
@@ -1091,7 +1069,7 @@ static int _shatter_mon_dice(const monster *mon)
         else if (mon->is_insubstantial())
             return 0;
         // 1/3 damage to fliers and slimes.
-        else if (mon->airborne() || mons_is_slime(mon))
+        else if (mon->airborne() || mons_is_slime(*mon))
             return 1;
         // 3/2 damage to ice.
         else if (mon->is_icy())
@@ -1200,8 +1178,6 @@ static int _shatter_walls(coord_def where, int pow, actor *agent)
 
         destroy_wall(where);
 
-        if (agent->is_player() && grid == DNGN_ORCISH_IDOL)
-            did_god_conduct(DID_DESTROY_ORCISH_IDOL, 8);
         if (agent->is_player() && feat_is_tree(grid))
             did_god_conduct(DID_KILL_PLANT, 1);
 
@@ -1504,7 +1480,7 @@ spret_type cast_irradiate(int powc, actor* who, bool fail)
         mpr("You erupt in a fountain of uncontrolled magic!");
     else
     {
-        simple_monster_message(who->as_monster(),
+        simple_monster_message(*who->as_monster(),
                                " erupts in a fountain of uncontrolled magic!");
     }
 
@@ -1529,12 +1505,7 @@ spret_type cast_irradiate(int powc, actor* who, bool fail)
     }, who->pos(), true, 8);
 
     if (who->is_player())
-    {
-        contaminate_player(1500 + random2(1500)); // on avg, a bit under 50% of
-                                                  // yellow contam
-                                                  // another cast might or
-                                                  // might not push you over
-    }
+        contaminate_player(1000 + random2(500));
     return SPRET_SUCCESS;
 }
 
@@ -1580,7 +1551,12 @@ static int _ignite_poison_clouds(coord_def where, int pow, actor *agent)
         return false;
 
     if (tracer)
+    {
+        // players just care if igniteable clouds exist
+        if (agent && agent->is_player())
+            return 1;
         return _ignite_tracer_cloud_value(where, agent);
+    }
 
     cloud->type = CLOUD_FIRE;
     cloud->decay = 30 + random2(20 + pow); // from 3-5 turns to 3-15 turns
@@ -1637,9 +1613,14 @@ static int _ignite_poison_monsters(coord_def where, int pow, actor *agent)
     mon->expose_to_element(BEAM_FIRE, damage);
 
     if (tracer)
+    {
+        // players don't care about magnitude, just care if enemies exist
+        if (agent && agent->is_player())
+            return mons_aligned(mon, agent) ? 0 : 1;
         return mons_aligned(mon, agent) ? -1 * damage : damage;
+    }
 
-    simple_monster_message(mon, " seems to burn from within!");
+    simple_monster_message(*mon, " seems to burn from within!");
 
     dprf("Dice: %dd%d; Damage: %d", dam_dice.num, dam_dice.size, damage);
 
@@ -1651,7 +1632,7 @@ static int _ignite_poison_monsters(coord_def where, int pow, actor *agent)
 
         // Monster survived, remove any poison.
         mon->del_ench(ENCH_POISON, true); // suppress spam
-        print_wounds(mon);
+        print_wounds(*mon);
     }
     else
     {
@@ -1798,26 +1779,17 @@ bool ignite_poison_affects(const actor* act)
  * @param fail          If it's a player spell, whether the spell fail chance
  *                      was hit (whether the spell will fail as soon as the
  *                      player chooses not to abort the casting)
- * @param mon_tracer    If it's a monster spell, whether the 'casting' is just
- *                      a tracer (a check to see if it's worth actually casting)
+ * @param mon_tracer    Whether the 'casting' is just a tracer (a check to see
+ *                      if it's worth actually casting)
  * @return              If it's a tracer, SPRET_SUCCESS if the spell should
- *                      be cast & SPRET_SUCCESS otherwise.
+ *                      be cast & SPRET_ABORT otherwise.
  *                      If it's a real spell, SPRET_ABORT if the player chose
  *                      to abort the spell, SPRET_FAIL if they failed the cast
  *                      chance, and SPRET_SUCCESS otherwise.
  */
-spret_type cast_ignite_poison(actor* agent, int pow, bool fail, bool mon_tracer)
+spret_type cast_ignite_poison(actor* agent, int pow, bool fail, bool tracer)
 {
-    if (agent->is_player())
-    {
-        if (maybe_abort_ignite())
-        {
-            canned_msg(MSG_OK);
-            return SPRET_ABORT;
-        }
-        fail_check();
-    }
-    else if (mon_tracer)
+    if (tracer)
     {
         // Estimate how much useful effect we'd get if we cast the spell now
         const int work = apply_area_visible([agent] (coord_def where) {
@@ -1829,6 +1801,16 @@ spret_type cast_ignite_poison(actor* agent, int pow, bool fail, bool mon_tracer)
         return work > 0 ? SPRET_SUCCESS : SPRET_ABORT;
     }
 
+    if (agent->is_player())
+    {
+        if (maybe_abort_ignite())
+        {
+            canned_msg(MSG_OK);
+            return SPRET_ABORT;
+        }
+        fail_check();
+    }
+
     targetter_los hitfunc(agent, LOS_NO_TRANS);
     flash_view_delay(
         agent->is_player()
@@ -2065,6 +2047,7 @@ bool setup_fragmentation_beam(bolt &beam, int pow, const actor *caster,
 
         case MONS_IRON_ELEMENTAL:
         case MONS_IRON_GOLEM:
+        case MONS_PEACEKEEPER:
         case MONS_WAR_GARGOYLE:
             beam.name       = "blast of metal fragments";
             beam.colour     = CYAN;
@@ -2080,6 +2063,12 @@ bool setup_fragmentation_beam(bolt &beam, int pow, const actor *caster,
             beam.damage.num = 3;
             break;
 
+        case MONS_SALTLING:
+            beam.name       = "blast of salt crystal fragments";
+            beam.colour     = WHITE;
+            beam.damage.num = 3;
+            break;
+
         case MONS_OBSIDIAN_STATUE:
         case MONS_ORANGE_STATUE:
         case MONS_CRYSTAL_GUARDIAN:
@@ -2162,9 +2151,6 @@ bool setup_fragmentation_beam(bolt &beam, int pow, const actor *caster,
     {
     // Stone and rock terrain
     case DNGN_ORCISH_IDOL:
-        if (!caster->is_player())
-            return false; // don't let monsters blow up orcish idols
-
         if (what && *what == nullptr)
             *what = "stone idol";
         // fall-through
@@ -2287,7 +2273,6 @@ spret_type cast_fragmentation(int pow, const actor *caster,
     bool should_destroy_wall = false;
     bool hole                = true;
     const char *what         = nullptr;
-    const dungeon_feature_type grid = grd(target);
 
     bolt beam;
 
@@ -2300,16 +2285,6 @@ spret_type cast_fragmentation(int pow, const actor *caster,
 
     if (caster->is_player())
     {
-        if (grid == DNGN_ORCISH_IDOL)
-        {
-            if (!yesno("Really insult Beogh by defacing this idol?",
-                       false, 'n'))
-            {
-                canned_msg(MSG_OK);
-                return SPRET_ABORT;
-            }
-        }
-
         bolt tempbeam;
         bool temp;
         setup_fragmentation_beam(tempbeam, pow, caster, target, false, true,
@@ -2366,11 +2341,6 @@ spret_type cast_fragmentation(int pow, const actor *caster,
 
     beam.explode(true, hole);
 
-    // Monsters shouldn't be able to blow up idols,
-    // but this check is here just in case...
-    if (caster->is_player() && grid == DNGN_ORCISH_IDOL)
-        did_god_conduct(DID_DESTROY_ORCISH_IDOL, 8);
-
     return SPRET_SUCCESS;
 }
 
@@ -2765,7 +2735,7 @@ spret_type cast_toxic_radiance(actor *agent, int pow, bool fail, bool mon_tracer
     }
     else if (mon_tracer)
     {
-        for (actor_near_iterator ai(agent, LOS_NO_TRANS); ai; ++ai)
+        for (actor_near_iterator ai(agent->pos(), LOS_NO_TRANS); ai; ++ai)
         {
             if (!_toxic_can_affect(*ai) || mons_aligned(agent, *ai))
                 continue;
@@ -2779,7 +2749,7 @@ spret_type cast_toxic_radiance(actor *agent, int pow, bool fail, bool mon_tracer
     else
     {
         monster* mon_agent = agent->as_monster();
-        simple_monster_message(mon_agent,
+        simple_monster_message(*mon_agent,
                                " begins to radiate toxic energy.");
 
         mon_agent->add_ench(mon_enchant(ENCH_TOXIC_RADIANCE, 1, mon_agent,
@@ -2811,38 +2781,20 @@ void toxic_radiance_effect(actor* agent, int mult)
         if (agent->is_monster() && mons_aligned(agent, *ai))
             continue;
 
-        int dam = roll_dice(1, 3 + pow / 25) * mult;
-
-        // Only applied if the player is also not the agent; Done early so that
-        // distance falloff won't frequently reduce damage to 1.
-        if (ai->is_player())
-            dam = dam * 5 / 2;
-
-        // Give monster OTR a weaker distance falloff than player OTR
-        if (agent->is_player())
-            dam = dam * 3 / (2 + ai->pos().distance_from(agent->pos()));
-        else
-            dam = dam * 9 / (8 + ai->pos().distance_from(agent->pos()));
-
+        int dam = roll_dice(1, 1 + pow / 20) * mult;
         dam = resist_adjust_damage(*ai, BEAM_POISON, dam);
 
         if (ai->is_player())
         {
-            // We take direct damage only if we're not the agent, but we
-            // still get poisoned
+            // We're affected only if we're not the agent.
             if (!agent->is_player())
             {
                 ouch(dam, KILLED_BY_BEAM, agent->mid,
                     "by Olgreb's Toxic Radiance", true,
                     agent->as_monster()->name(DESC_A).c_str());
 
-                poison_player(dam * 4 / 3, agent->name(DESC_A), "toxic radiance",
-                              false);
-            }
-            else
-            {
                 poison_player(roll_dice(2, 3), agent->name(DESC_A),
-                              "toxic radiance", true);
+                              "toxic radiance", false);
             }
         }
         else
@@ -2927,7 +2879,10 @@ void handle_searing_ray()
     dec_mp(1);
 
     if (++you.attribute[ATTR_SEARING_RAY] > 3)
+    {
+        mpr("You finish channeling your searing ray.");
         end_searing_ray();
+    }
 }
 
 void end_searing_ray()
@@ -2954,7 +2909,7 @@ static bool _player_glaciate_affects(const actor *victim)
     if (!mon) // player
         return true;
 
-    return !mons_is_projectile(mon)
+    return !mons_is_projectile(*mon)
             && (!mons_is_avatar(mon->type) || !mons_aligned(&you, mon));
 }
 
@@ -3145,7 +3100,7 @@ spret_type cast_scattershot(const actor *caster, int pow, const coord_def &pos,
         if (!mons || !mons->alive() || !you.can_see(*mons))
             continue;
 
-        print_wounds(mons);
+        print_wounds(*mons);
     }
 
     return SPRET_SUCCESS;
