@@ -97,15 +97,10 @@ static void _player_change_level(level_id lev)
     you.where_are_you = lev.branch;
 }
 
-static bool _marker_vetoes_level_change()
-{
-    return marker_vetoes_operation("veto_level_change");
-}
-
-static void _maybe_destroy_trap(const coord_def &p)
+static void _maybe_destroy_shaft(const coord_def &p)
 {
     trap_def* trap = trap_at(p);
-    if (trap)
+    if (trap && trap->type == TRAP_SHAFT)
         trap->destroy(true);
 }
 
@@ -203,7 +198,7 @@ static void _clear_golubria_traps()
 
 static void _clear_prisms()
 {
-    for (auto &mons : menv)
+    for (auto &mons : menv_real)
         if (mons.type == MONS_FULMINANT_PRISM)
             mons.reset();
 }
@@ -442,7 +437,7 @@ static level_id _travel_destination(const dungeon_feature_type how,
         {
             if (known_shaft)
                 mpr("The shaft disappears in a puff of logic!");
-            _maybe_destroy_trap(you.pos());
+            _maybe_destroy_shaft(you.pos());
             return dest;
         }
 
@@ -482,7 +477,7 @@ static level_id _travel_destination(const dungeon_feature_type how,
                 mpr("The strain on the space-time continuum destroys the "
                     "shaft!");
             }
-            _maybe_destroy_trap(you.pos());
+            _maybe_destroy_shaft(you.pos());
             return dest;
         }
 
@@ -492,8 +487,6 @@ static level_id _travel_destination(const dungeon_feature_type how,
                                     + shaft_dest.describe() + ".");
         }
 
-        handle_items_on_shaft(you.pos(), false);
-
         string howfar;
         if (shaft_depth > 1)
             howfar = make_stringf(" for %d floors", shaft_depth);
@@ -504,7 +497,7 @@ static level_id _travel_destination(const dungeon_feature_type how,
 
         // Shafts are one-time-use.
         mpr("The shaft crumbles and collapses.");
-        _maybe_destroy_trap(you.pos());
+        _maybe_destroy_shaft(you.pos());
     }
 
     // Maybe perform the entry sequence (we check that they have enough runes
@@ -523,10 +516,6 @@ static level_id _travel_destination(const dungeon_feature_type how,
         break;
     }
 
-    // Bail if any markers veto the move.
-    if (_marker_vetoes_level_change())
-        return dest;
-
     // Markers might be deleted when removing portals.
     const string dst = env.markers.property_at(you.pos(), MAT_ANY, "dst");
 
@@ -740,6 +729,10 @@ void floor_transition(dungeon_feature_type how,
                 else if (branch != BRANCH_ABYSS) // too many messages...
                     mprf("Welcome to %s!", branches[branch].longname);
 
+                const string noise_desc = branch_noise_desc(branch);
+                if (!noise_desc.empty())
+                    mpr(noise_desc);
+
                 const string rune_msg = branch_rune_desc(branch, true);
                 if (!rune_msg.empty())
                     mpr(rune_msg);
@@ -1020,8 +1013,12 @@ static void _update_level_state()
     if (_any_glowing_mold())
         env.level_state |= LSTATE_GLOW_MOLD;
     for (monster_iterator mon_it; mon_it; ++mon_it)
-        if (mons_allows_beogh(*mon_it))
+    {
+        if (mons_allows_beogh(**mon_it))
             env.level_state |= LSTATE_BEOGH;
+        if (mon_it->has_ench(ENCH_STILL_WINDS))
+            env.level_state |= LSTATE_STILL_WINDS;
+    }
     for (rectangle_iterator ri(0); ri; ++ri)
         if (grd(*ri) == DNGN_SLIMY_WALL)
             env.level_state |= LSTATE_SLIMY_WALL;
