@@ -71,8 +71,6 @@ static uint8_t _jewellery_type_from_artefact_prop(const string &s
         return AMU_RAGE;
     if (s == "Harm")
         return AMU_HARM;
-    if (s == "Dismiss")
-        return AMU_DISMISSAL;
     if (s == "Gourm")
         return AMU_THE_GOURMAND;
     if (s == "Inacc")
@@ -92,8 +90,6 @@ static uint8_t _jewellery_type_from_artefact_prop(const string &s
         return RING_ICE;
     if (s == "+/*Tele")
         return RING_TELEPORTATION;
-    if (s == "SustAt" || s == "SustAb")
-        return RING_SUSTAIN_ATTRIBUTES;
     if (s == "Wiz")
         return RING_WIZARDRY;
     if (s == "SInv")
@@ -392,11 +388,11 @@ bool chardump_parser::_check_skill(const vector<string> &tokens)
     double amount = atof(tokens[2].c_str());
     set_skill_level(skill, amount);
     if (tokens[0] == "+")
-        you.train[skill] = 1;
+        you.train[skill] = TRAINING_ENABLED;
     else if (tokens[0] == "*")
-        you.train[skill] = 2;
+        you.train[skill] = TRAINING_FOCUSED;
     else
-        you.train[skill] = 0;
+        you.train[skill] = TRAINING_DISABLED;
 
     redraw_skill(skill);
 
